define('framework/dataGrid/DataGridModule', [
    'angular',
    'jquery',
    'underscore',
    'artTmpl',
    'framework/datasource/DataSourceModule',
    'framework/colSettings/ColSettingsModule',
    'framework/clickbox/GillionClickBoxModule'
], function (angular, $, _, artTmpl) {
    var CONSTANTS = {
        CHECK_ROW_WIDTH: 40,
        SPACE_CELL_WIDTH: 19,
        MIN_CELL_WIDTH: 50,
        REFRESH_ROWS: '$grid-refresh-rows'
    }, isIE = navigator.appName == "Microsoft Internet Explorer";

    angular.module('DataGridModule', ['DataSourceModule', 'ColSettingsModule', 'GillionClickBoxModule'])
        .directive('gDataGrid', function ($window, $document, $parse, $compile, $filter, $dataSourceManager, ColSettings, Arrays, Functions, LocalStorages) {
            var GridProto = Grid.prototype,
                ORDER = {
                    ASC: 'asc',
                    DESC: 'desc'
                };

            function Grid(scope, element, attrs) {
                var me = this;
                me.scope = scope;
                me.element = element;
                me.attrs = attrs;
                me.width = element.width();
                me.hash = scope.$id;
                me.widthStyleBlock = element.children('style')[0];

                me.cellValueGetterCache = {};
                me.scopeEventCache = {};
                scope.columns = me.columns = [];
                if (attrs.$columns) {
                    scope.$columns = me.columns;
                }
                me.footCellIndex = 0;
                /**
                 * @type {{String: [Number, Number]}}
                 */
                me.requiredClassNameColSpanRange = {};
                me.actions = {};

                me.tbody = element.find('table.table-body tbody')[0];
                me._disabledCheckboxRowIndexes = [];

                me.$headerBox = me.element.children('div.grid-head');
                me.$colResizeLine = me.element.find('div.col-resize-line');
            }

            GridProto._registerHeaderDraggable = function () {
                var me = this;
                if (!me.scope.colSettingsKey) {
                    return;
                }
                var $headerBox = me.$headerBox,
                    $colResizeLine = me.$colResizeLine;
                $headerBox.mousemove(function ($event) {
                    var target = $event.target,
                        $th = angular.element(target).closest('th'),
                        width, offset, eX, inRightEdge, dragging;
                    if ($th.length > 0) {
                        width = $th.width();
                        offset = $th.offset();
                        eX = $event.clientX;
                        inRightEdge = (eX - offset.left) > width;
                        dragging = angular.isDefined($headerBox.data('_start_x'));
                        if (inRightEdge || dragging) {
                            $headerBox.addClass('resizeable');
                            if (dragging) {
                                $colResizeLine.css('left', $event.clientX + me.element.children('div.grid-body').scrollLeft() - 7);
                                if (isIE) {
                                    $document.onselectstart = function () {
                                        return false;
                                    };
                                }
                            }
                        } else {
                            $headerBox.removeClass('resizeable');
                        }
                    }
                });
                $headerBox.mousedown(function ($event) {
                    var target = $event.target,
                        $th = angular.element(target).closest('th');
                    if ($th.length > 0 && $headerBox.hasClass('resizeable')) {
                        me._startColResize($event, $th);
                        angular.element($document).one('mouseup', function ($event) {
                            me._finishColResize($event, $th);
                        });
                    }
                });
            };

            GridProto._startColResize = function ($event) {
                var me = this,
                    $headerBox = me.$headerBox,
                    $colResizeLine = me.$colResizeLine;
                $colResizeLine.css('left', $event.clientX + me.element.children('div.grid-body').scrollLeft() - 7);
                $colResizeLine.show();
                $headerBox.data('_start_x', $event.clientX);
            };

            GridProto._finishColResize = function ($event, $th) {
                var me = this,
                    $colResizeLine = me.$colResizeLine,
                    $headerBox = me.$headerBox,
                    startX = $headerBox.data('_start_x'),
                    endX = $event.clientX,
                    added = endX - startX,
                    headerScope = $th.isolateScope(),
                    col = headerScope.column,
                    colSettings = me._getColSettings(),
                    oIdx = _.findIndex(me.__originalCols, col),
                    colSetting = _.findWhere(colSettings, {oIdx: oIdx}),
                    $styleBlock = me.element.children('style'),
                    colWidthClassName = col.colWidthClassName,
                    oldStyleContent = $styleBlock.html(),
                    newStyleContent;

                colSetting.width = $th.width() + (endX - startX);
                LocalStorages.set(me.scope.colSettingsKey, colSettings);

                newStyleContent = oldStyleContent.replace(getColClassReg(colWidthClassName), function (block, width) {
                    var newWidth = Number(width) + added;
                    newWidth = _.isNaN(newWidth) ? CONSTANTS.MIN_CELL_WIDTH : newWidth;
                    newWidth = Math.max(newWidth, CONSTANTS.MIN_CELL_WIDTH);
                    return block.replace(width, newWidth);
                });

                $styleBlock.html(newStyleContent);

                $colResizeLine.hide();
                $headerBox.removeData('_start_x');
                if (isIE) {
                    delete $document.onselectstart;
                }
            };

            function getColClassReg(colWidthClassName){
                var matched = colWidthClassName.match(/grid-(.+)-col-(\d+)/),
                    colIndex = matched[2];
                return new RegExp('(?:\\' + colWidthClassName + '|\\\.grid-' + matched[1] + '-col-[0-' + colIndex + ']-[' + (colIndex + 1) + '-9])\\\{width:([\\\d\\\.]+)px;}', 'g');
            }

            /**
             * @public
             * @param recordOrIndex {Object/Number} 记录或记录在表格中的 index
             */
            GridProto.saveRecord = function (recordOrIndex) {
                var me = this,
                    recordIndex;
                if (_.isObject(recordOrIndex)) {
                    recordIndex = _.chain(me.scope.source).indexOf(recordOrIndex).value();
                } else if (_.isNumber(recordOrIndex)) {
                    recordIndex = recordOrIndex;
                }
                if (_.isNumber(recordIndex)) {
                    me.renderRowTitle(recordIndex);
                    me._reRenderRow(recordIndex);
                }
            };

            /**
             * @see Grid#saveRecord
             * @private
             *  重新生成行 `title`,
             *  <b>请不要使用这个方法， 这个方法被修改为私有了。</b>
             * @param param {Number/HTMLTableRowElement/function(rowData:Object):Boolean} 行号或行DOM
             */
            GridProto.renderRowTitle = function (param) {
                var me = this,
                    columns = me.columns,
                    source = me.scope.source,
                    trs = me.tbody.rows,
                    row, $row, rowIndex, rowData;
                if (angular.isNumber(param)) {
                    rowIndex = param;
                } else if (angular.isFunction(param)) {
                    rowIndex = _.findIndex(source, param);
                } else if (($row = angular.element(param)).is('tr')) {
                    rowIndex = $row[0].rowIndex;
                }
                if (row && rowIndex) {
                    row = trs[param];
                    rowData = source[rowIndex];
                    _.each(row.cells, function (i, cell) {
                        var columnIndex = me._getColumnIndex(cell),
                            column = columns[columnIndex];
                        me._renderCellTitle(rowData, column, cell);
                    });
                }
            };

            /**
             * @param rowData 行数据
             * @param column 列对象
             * @param $cellDom {HTMLTableCellElement} 列`dom`
             * @private
             */
            GridProto._renderCellTitle = function (rowData, column, $cellDom) {
                var me = this;
                if (!column.disableTitle) {
                    var cellValue = me.getCellValue(rowData, column);
                    angular.element($cellDom).attr('title', cellValue);
                }
            };

            GridProto._addColumn = function (column) {
                return this.columns.push(column);
            };

            GridProto._getCellValue = function (row, column) {
                var me = this,
                    cellValueGetterCache = me.cellValueGetterCache;
                if (!cellValueGetterCache[column.data]) {
                    cellValueGetterCache[column.data] = me.generateCellValueGetter(column);
                }
                return cellValueGetterCache[column.data](row);
            };

            GridProto._generateCellValueGetter = function (column) {
                if (column.data) {
                    return $parse(column.data);
                }
                return angular.noop;
            };

            GridProto._getSortName = function (sortingIcon) {
                var me = this,
                    columnIndex = me._getColumnIndex(sortingIcon.closest('th'));
                return me.columns[columnIndex].sortable;
            };

            /**
             * @param $event
             * @param $sortingIcon
             */
            GridProto._specifySort = function ($event, $sortingIcon) {
                var me = this,
                    $header = $sortingIcon.closest('th'),
                    column = $header.isolateScope().column,
                    newSortName = column.field,
                    newDirection = $sortingIcon.hasClass('caret-up') ? ORDER.ASC : ORDER.DESC;
                newDirection = me._desortIfSameAsOld(newSortName, newDirection);
                me._doSort(newSortName, newDirection);
            };

            GridProto._toggleSort = function ($event) {
                var me = this,
                    $target = angular.element($event.target),
                    $header = $target.closest('th'),
                    isolateScope = $header.isolateScope(),
                    column, newSortName, oldSortName, oldSortDirection, newDirection;
                if (!(isolateScope && isolateScope.column && isolateScope.column.sortable)) return;

                column = isolateScope.column;
                newSortName = column.field;
                oldSortName = me.sortName;
                oldSortDirection = me.sortDirection;
                newDirection = ORDER.ASC;
                if (newSortName === oldSortName) {
                    if (oldSortDirection === ORDER.ASC) {
                        newDirection = ORDER.DESC;
                    } else if (oldSortDirection === ORDER.DESC) {
                        newDirection = undefined;
                    }
                }
                me._doSort(newSortName, newDirection);
            };

            GridProto._doSort = function (sortName, sortDirection) {
                if (this.__rendered !== true) return;
                var me = this,
                    scope = me.scope,
                    onBeforeSortColumn = scope.onBeforeSortColumn,
                    onSortColumn = scope.onSortColumn,
                    dataSource = scope.dataSource,
                    sortEventParam = {
                        grid: this,
                        sortName: sortName,
                        sortDirection: sortDirection
                    };
                if (onBeforeSortColumn(sortEventParam) !== false) {
                    dataSource.sortName = me.sortName = sortName;
                    dataSource.sortDirection = me.sortDirection = sortDirection;
                    var allowAutoLoad = dataSource.$scope.allowAutoLoad;
                    if (!(allowAutoLoad === 'true' || allowAutoLoad === true)) {
                        dataSource.doRequestData();
                    }
                    onSortColumn(sortEventParam);
                }
            };

            GridProto._desortIfSameAsOld = function (newSortName, newDirection) {
                var me = this,
                    oldSortName = me.sortName,
                    oldSortDirection = me.sortDirection;
                if (oldSortName === newSortName && oldSortDirection === newDirection) {
                    return undefined;
                }
                return newDirection;
            };

            GridProto.isAllRowsChecked = function () {
                return this.scope.allChecked;
            };

            GridProto._isAllRowsChecked = function () {
                var me = this,
                    scope = me.scope,
                    disabledRows = me._getDisabledRows(),
                    disabledCount = disabledRows.length,
                    sourceLen = scope.source.length,
                    enableCount = sourceLen - disabledCount,
                    checkedEnableCount = _.difference(scope.checkedRows, disabledRows).length;
                return checkedEnableCount === enableCount;
            };

            GridProto._getDisabledRows = function () {
                return _.chain(this.scope.source)
                    .map(_disabledLocalsTrans)
                    .filter(this._disabledRowPredicate())
                    .map(_.property("row"))
                    .value();
            };

            function _disabledLocalsTrans(row, index) {
                return {
                    row: row,
                    rowIndex: index
                };
            }

            GridProto._disabledRowPredicate = function () {
                var me = this,
                    express = me.checkboxesDisabledExpress;
                if (express) {
                    if (/\([$\w]+(?:,\s*[$\w]+)*\)$/.test(express)) {
                        return me._getScopeEvent(express);
                    } else {
                        return $parse(express);
                    }
                }
                return _.constant(false);
            };

            GridProto._toggleCellChecked = function ($event) {
                var $target = angular.element($event.target),
                    $checkbox = $target.closest('.form-clickbox'),
                    me, scope, source, checkedRows, rowIndex, row, checked;
                if ($checkbox.is('[disabled=disabled]')) return;
                me = this;
                scope = me.scope;
                source = scope.source;
                checkedRows = scope.checkedRows;
                rowIndex = $checkbox.closest('tr')[0].rowIndex;
                row = source[rowIndex];
                checked = _.contains(checkedRows, row);

                $checkbox.toggleClass('selected', !checked);
                if (checked) {
                    Arrays.remove(checkedRows, row);
                } else {
                    me.scope.checkedRows.push(source[rowIndex]);
                }
                // binded
                me.scope.allChecked = me._isAllRowsChecked();
                me.checkRowByContrl = true;
            };

            GridProto._toggleAllChecked = function () {
                var me = this,
                    trs = me.tbody.rows,
                    scope = me.scope,
                    source = scope.source,
                    checkedRows = scope.checkedRows,
                    disabledRows = me._getDisabledRows(),
                    enabledRows = Arrays.subtract(source, disabledRows),
                    i, len, record;
                checkedRows.length = 0;
                if (scope.allChecked) {
                    Arrays.pushAll(enabledRows, checkedRows);
                }
                for (i = 0, len = source.length; i < len; i++) {
                    record = source[i];
                    if (_.contains(enabledRows, record)) {
                        angular.element('td:first-child > div.form-clickbox', trs[i]).toggleClass('selected', scope.allChecked);
                    }
                }
                me.checkRowByContrl = true;
            };

            GridProto._isChecked = function (record) {
                var scope = this.scope;
                return scope.allChecked || _.contains(scope.checkedRows, record);
            };

            GridProto.isChecked = GridProto._isChecked;

            GridProto._getEnableCheckboxes$ = function () {
                return angular.element('td:first-child > div[disabled!=disabled]', this.tbody);
            };

            GridProto._syncCheckRowsByOuter = function () {
                var me = this,
                    scope = me.scope,
                    checkedRows = scope.checkedRows,
                    source = scope.source;
                if (angular.isArray(source) && source.length > 0 && angular.isArray(checkedRows)) {
                    angular.forEach(me.tbody.rows, function (tr) {
                        var rowIndex = tr.rowIndex,
                            rowData = source[rowIndex],
                            $checkbox = angular.element(tr.firstChild.firstChild);
                        if (_.contains(checkedRows, rowData)) {
                            $checkbox.addClass('selected');
                        } else {
                            $checkbox.removeClass('selected');
                        }
                    });
                    me.scope.allChecked = me._isAllRowsChecked();
                }
            };

            GridProto._headClick = function ($event) {
                var me = this,
                    target = $event.target,
                    $target = angular.element(target),
                    $headTab = me.element.children('div.grid-head');
                if ($target.is('span.caret')) {
                    me._specifySort($event, $target);
                } else if (!$headTab.hasClass('resizeable')) {
                    me._toggleSort($event);
                }
                if ($target.is('a.fi') && $target.parent().is('.form-clickbox')) {
                    me._toggleAllChecked();
                }
            };

            GridProto._rowClick = function ($event, $row) {
                var me = this,
                    scope = me.scope,
                    rowIndex = $row[0].rowIndex,
                    oldSelected = scope.selectedRow,
                    selectedRecord = scope.source[rowIndex],
                    isSelected = !(scope.canToggleSelected === 'true' && oldSelected === selectedRecord),
                    onBeforeSelect = scope.onBeforeSelect,
                    onSelect = scope.onSelect,
                    selectParam = {
                        $event: $event,
                        record: selectedRecord,
                        grid: me,
                        $row: $row,
                        isSelected: isSelected
                    };

                if (onBeforeSelect(selectParam) !== false) {
                    scope.selectedRow = isSelected ? selectedRecord : undefined;
                    if (me.__oldSelectedRow) {
                        me.__oldSelectedRow.removeAttr('active')
                    }
                    if (isSelected) {
                        $row.attr('active', true);
                    }
                    me.__oldSelectedRow = $row;
                    onSelect(selectParam);
                }

            };

            GridProto._rowDbClick = function ($event, $row) {
                var me = this,
                    scope = me.scope,
                    selectedRecord = scope.selectedRow,
                    onBeforeRowDbclick = scope.onBeforeRowDbclick,
                    onRowDbclick = scope.onRowDbclick,
                    eventParam = {
                        $event: $event,
                        record: selectedRecord,
                        grid: me,
                        $row: $row
                    };

                if (onBeforeRowDbclick(eventParam) !== false) {
                    onRowDbclick(eventParam);
                }
            };

            GridProto._cellDbClick = function ($event, $cell) {
                var me = this,
                    scope = me.scope,
                    selectedRecord = scope.selectedRow,
                    columnIndex = me._getColumnIndex($cell),
                    column = me.columns[columnIndex],
                    globalBeforeCellDbclick = scope.onBeforeCellDbclick,
                    globalCellDbclick = scope.onCellDbclick,
                    onBeforeCellDbClick = angular.isFunction(column.onBeforeCellDbclick) ? column.onBeforeCellDbclick : angular.noop,
                    onCellDbClick = angular.isFunction(column.onCellDbclick) ? column.onCellDbclick : angular.noop,
                    eventParam = {
                        $event: $event,
                        record: selectedRecord,
                        grid: me,
                        $cell: $cell,
                        column: column
                    };

                if (globalBeforeCellDbclick(eventParam) !== false) {
                    globalCellDbclick(eventParam);
                }
                if (onBeforeCellDbClick(eventParam) !== false) {
                    onCellDbClick(eventParam);
                }
            };

            /**
             * 子指令scope定义方法属性时， 指定到grid外scope的某个方法， 通过本方法获取
             * @param eventDefineAttrVal 方法属性指定的值
             * @return {*}
             */
            GridProto._getScopeEvent = function (eventDefineAttrVal) {
                var me = this,
                    scopeEventCache = me.scopeEventCache,
                    gridOuterScope = me.scope.$parent;
                if (!!eventDefineAttrVal) {
                    if (!scopeEventCache[eventDefineAttrVal]) {
                        var parentGet = $parse(eventDefineAttrVal);
                        return function (locals) {
                            return parentGet(gridOuterScope, locals);
                        };
                    }
                    return scopeEventCache[eventDefineAttrVal];
                }
                return angular.noop;
            };

            GridProto._renderCellsStyle = function () {
                var me = this,
                    element = me.element,
                    allColWidths = me._getAllColWidths(),
                    colWidthClassBlocks = me._getColWidthClassBlocks(allColWidths),
                    requiredSpanColClassBlocks = me._getRequiredSpanColClassBlocks(allColWidths),
                    allColWidthClassBlocks = colWidthClassBlocks.concat(requiredSpanColClassBlocks),
                    cellWidthStyleBlockContent = _.reduce(allColWidthClassBlocks, Functions.sum),
                    $newCellStyleBlock = angular.element('<style>' + cellWidthStyleBlockContent + '</style>');
                element.children('style').remove();
                element.append($newCellStyleBlock);
            };

            /**
             * 获取列的宽度样式片段
             *
             * @param allColWidths {Array<Number>} 所有列的宽度
             * @return {Array<String>} 宽度样式片段
             */
            GridProto._getColWidthClassBlocks = function (allColWidths) {
                var me = this,
                    colWidthClassBlocks = [],
                    colWidthClassName, colWidthClassBlock;
                angular.forEach(allColWidths, function (colWidth, colIndex) {
                    colWidthClassName = me._getColWidthClassName(colIndex);
                    colWidthClassBlock = _toWidthClassBlock(colWidthClassName, colWidth);
                    colWidthClassBlocks.push(colWidthClassBlock);
                });
                return colWidthClassBlocks;
            };

            /**
             * 获取跨列列的宽度样式片段
             *
             * @param allColWidths {Array<Number>} 所有列的宽度
             * @return {Array<String>} 宽度样式片段
             */
            GridProto._getRequiredSpanColClassBlocks = function (allColWidths) {
                var me = this,
                    spanColClassBlocks = [],
                    requiredClassNameColSpanRange = me.requiredClassNameColSpanRange,
                    start, end, spanColWidths, spanColWidthSum, spanColWidthClassBlock;
                angular.forEach(requiredClassNameColSpanRange, function (spanRange, requiredClassName) {
                    start = spanRange[0];
                    end = spanRange[1];
                    spanColWidths = allColWidths.slice(start, end);
                    spanColWidthSum = Arrays.reduce(spanColWidths, Functions.sum);
                    spanColWidthClassBlock = _toWidthClassBlock(requiredClassName, spanColWidthSum);
                    spanColClassBlocks.push(spanColWidthClassBlock);
                });
                return spanColClassBlocks;
            };

            function _toWidthClassBlock(className, width) {
                return '.' + className + '{width:' + width + 'px;}'
            }

            /**
             * 将 Array 中未定义的位置用余数的平均值填充, 如果平均值比最小值小， 用最小值填充
             * @example paddingWidthSpentAvg([1, undefined, 3, 4, undefined], 20, 2) -> [1, 6, 3, 4, 6]
             * @example paddingWidthSpentAvg([1, undefined, 3, 4, undefined], 20, 7) -> [1, 7, 3, 4, 7]
             * @param array {Array<Number>} 需要填充的数组
             * @param total {Number} 总值
             * @param min {Number} 最小值
             */
            function paddingWithSpentAvg(array, total, min) {
                var definedEls = Arrays.filter(array, angular.isDefined),
                    definedSum = Arrays.isNotEmptyArray(definedEls) ? Arrays.reduce(definedEls, Functions.sum) : 0,
                    undefinedCount = array.length - definedEls.length,
                    spent = total - definedSum,
                    spentAvg = spent / undefinedCount,
                    padding = Math.max(spentAvg, min);
                angular.forEach(array, function (el, index) {
                    if (angular.isUndefined(el)) {
                        array[index] = padding;
                    }
                });
            }

            /**
             * 获取所有列的宽度， 包括勾选列和补白列
             *
             * @return {Array<Number>} 所有列的宽度
             */
            GridProto._getAllColWidths = function () {
                var me = this,
                    __originalCols = me.__originalCols || me.columns,
                    gridContentWidth = me._getContentWidth(),
                    columnWidths;
                var colSettings = me._getColSettings();
                columnWidths = Arrays.transform(__originalCols, function (col) {
                    var oIdx = _.findIndex(me.__originalCols, col), colSetting;
                    if (oIdx && _.isArray(colSettings)) {
                        colSetting = _.findWhere(colSettings, {oIdx: oIdx});
                        if (colSetting && colSetting.width) {
                            return colSetting.width;
                        }
                    }
                    return me._getSingleColWidth(col.widthDef, gridContentWidth);
                });
                paddingWithSpentAvg(columnWidths, gridContentWidth, CONSTANTS.MIN_CELL_WIDTH);
                if (me.hasCheckbox) {
                    columnWidths.unshift(CONSTANTS.CHECK_ROW_WIDTH);
                }
                if (me._hasVerticalScroll()) {
                    columnWidths.push(CONSTANTS.SPACE_CELL_WIDTH);
                }
                return columnWidths;
            };

            GridProto._hasVerticalScroll = function () {
                var me = this,
                    $gridBody = me.element.find('.grid-body'),
                    $bodyTable = $gridBody.children('.table-body');
                return $gridBody.width() < $bodyTable.width();
            };

            /**
             * 获取表格单列宽度
             * @param columnWidthDef {String} 列指令的宽度定义字符串 例: 100 、100px 、 10%
             * @param gridContentWidth {Number} 表格主体宽度, 即数据展示部分的宽度
             * @return {number} 列宽度
             */
            GridProto._getSingleColWidth = function (columnWidthDef, gridContentWidth) {
                var widthPercent;
                if (!!columnWidthDef) {
                    if (/^[\d\.]+%$/.test(columnWidthDef)) {
                        widthPercent = Number(columnWidthDef.substring(0, columnWidthDef.length - 1));
                        return gridContentWidth * widthPercent / 100;
                    } else if (/^[\d\.]+px$/.test(columnWidthDef)) {
                        return Number(columnWidthDef.substring(0, columnWidthDef.length - 2));
                    } else if (/^[\d\.]+$/.test(columnWidthDef)) {
                        return Number(columnWidthDef);
                    }
                }
            };

            /**
             * 获取表格主体部分宽度，即数据展示部分的总宽度， 为表格外层div宽度减去勾选列和空白列的宽度
             * @private
             */
            GridProto._getContentWidth = function () {
                var me = this,
                    gridWidth = me.element.width(),
                    gridContentWidth = gridWidth - 2;
                if (me.hasCheckbox) {
                    gridContentWidth -= CONSTANTS.CHECK_ROW_WIDTH;
                }
                if (me._hasVerticalScroll()) {
                    gridContentWidth -= CONSTANTS.SPACE_CELL_WIDTH
                }
                return gridContentWidth;
            };

            GridProto._registerFootCellWidthClass = function (span) {
                var me = this,
                    startColIndex = me.footCellIndex,
                    colspan = isNaN(span) ? 1 : span,
                    endColIndex = startColIndex + colspan,
                    colWidthClassName = me._getColWidthClassName(startColIndex);
                if (colspan > 1) {
                    colWidthClassName += ('-' + endColIndex);
                    me.requiredClassNameColSpanRange[colWidthClassName] = [startColIndex, endColIndex];
                }
                me.footCellIndex += colspan;
                return colWidthClassName;
            };

            /**
             * @private
             * @param startColIndex {Number} 开始的`column`的`index`
             * @return {string}
             */
            GridProto._getColWidthClassName = function (startColIndex) {
                return 'grid-' + this.hash + '-col-' + startColIndex;
            };

            GridProto.startEdit = function (rowIndex) {
                var me = this,
                    scope = me.scope,
                    outerScope = scope.$parent,
                    columns = me.columns,
                    formName = me.formName,
                    rowScope, tr, row;
                if (!me.__editingRowIndex) {
                    row = me.scope.source[rowIndex];
                    tr = me.tbody.rows[rowIndex];
                    rowScope = outerScope.$new();
                    rowScope.row = row;
                    rowScope.rowIndex = rowIndex;
                    rowScope.grid = me;
                    angular.forEach(tr.cells, function (cell) {
                        var $cell = angular.element(cell),
                            columnIndex = me._getColumnIndex($cell),
                            column = columns[columnIndex],
                            $editorPlace = $cell.children('[data-role=editor]'),
                            $displayPlace = $cell.children('[data-role=display]'),
                            columnEditorTmpl = me._getEditorTmpl(column),
                            cellTmplScope, $editor;

                        if (columnEditorTmpl) {
                            cellTmplScope = angular.extend(rowScope.$new(), {
                                grid: me,
                                columnIndex: columnIndex,
                                column: column
                            });
                            if (formName) {
                                cellTmplScope[formName] = scope[formName];
                            }
                            $editor = angular.element(columnEditorTmpl);
                            $editor.attr({
                                'ng-model': 'row.' + column.field,
                                'name': column.field
                            });
                            if (me.scope.validatorName) {
                                $editor.attr('g-field-validator', '');
                                $editor.data('$formController', me.formController);
                                $editor.data('$gValidatorController', me.gValidatorController);
                            }
                            $displayPlace.hide();
                            $editorPlace.append($editor);
                            $compile($editor)(cellTmplScope);
                        }
                    });
                    me.__editingRowIndex = rowIndex;
                }
            };

            GridProto.finishEdit = function () {
                var me = this,
                    formController = me.formController,
                    editingRowIdx;
                if (angular.isNumber(me.__editingRowIndex)) {
                    editingRowIdx = me.__editingRowIndex;
                    if (formController) {
                        formController.verify();
                        if (!formController.$valid) return;
                    }
                    me._reRenderRow(editingRowIdx);
                    delete  me.__editingRowIndex;
                }
            };

            GridProto._reRenderRow = function (rowIndex) {
                var me = this,
                    tbody = me.tbody,
                    trs = tbody.rows,
                    oldTr = trs[rowIndex];
                angular.element(oldTr).replaceWith(me._getCompiledRowTmpl()({
                    record: me.scope.source[rowIndex],
                    rowIndex: rowIndex,
                    grid: me
                }));
                $compile(trs[rowIndex])(me.scope);
            };

            GridProto._getEditorTmpl = function (column) {
                var me = this,
                    columnTmplMap = {};
                GridProto._getEditorTmpl = function (column) {
                    if (column) {
                        var field = column.field,
                            columnTmpl;
                        if (field) {
                            if (column.canEdit) {
                                if (!columnTmplMap.hasOwnProperty(field)) {
                                    if (column.editorTmpl) {
                                        columnTmplMap[field] = column.editorTmpl;
                                    } else if (column.editable) {
                                        switch (column.editable) {
                                            case 'text':
                                                columnTmplMap[field] = '<input type="text" class="form-text">';
                                                break;
                                            case 'number-spinner':
                                                columnTmplMap[field] = '<g-number-spinner></g-number-spinner>';
                                                break;
                                            case 'checkbox':
                                                columnTmplMap[field] = '<g-checkbox></g-checkbox>';
                                                break;
                                        }
                                    }
                                }
                                // 如果能编辑， 返回模板
                                columnTmpl = columnTmplMap[field];
                                if (!columnTmpl) {
                                    column.canEdit = false;
                                } else {
                                    return columnTmpl;
                                }
                            }
                        }
                    }
                };
                me._getEditorTmpl(column);
            };

            /**
             *
             * @param cellScope.rowIndex
             * @param cellScope.column
             * @param cellScope.row
             * @param cellScope.columnIndex
             * @returns {void|*}
             */
            GridProto._getCellTmplScope = function (cellScope) {
                var me = this,
                    scope = me.scope,
                    gridOuterScope = scope.$parent,
                    rowIndex = cellScope.rowIndex,
                    formName = me.formName,
                    cellTmplScope = angular.extend(gridOuterScope.$new(), {
                        grid: me,
                        column: cellScope.column,
                        rowIndex: rowIndex,
                        columnIndex: cellScope.columnIndex,
                        row: cellScope.row
                    });
                if (formName) {
                    cellTmplScope[formName] = scope[formName];
                }
                return cellTmplScope;
            };

            /**
             * 重置表格状态
             * 包括：   1、 全选状态
             *        2、 勾选行
             *        3、 选中行
             */
            GridProto._reset = function () {
                var me = this,
                    scope = me.scope;
                // 清空选中行
                scope.allChecked = false;
                scope.selectedRow = scope.$selectedRow = undefined;
                if (_.isArray(scope.$checkedRows)) {
                    scope.$checkedRows.length = 0;
                }
            };

            GridProto._getRowTmpl = function () {
                var me = this, columnsTmpl;
                if (!me.rowTmpl) {
                    columnsTmpl = _
                        .chain(me.columns)
                        .filter(angular.isDefined)
                        .map(function (column) {
                            if(!column){
                                return;
                            }
                            if (column.hide === true) return '';
                            var cellTmpl = '<td ';
                            if (column.gItemClass) {
                                cellTmpl += 'render-cell-class outer-scope="grid.scope.$parent" '
                            }
                            cellTmpl += 'class="' + column.colWidthClassName + ' ' + column.cellAlignClass + '"';
                            if (column.tmpl) {
                                cellTmpl += 'cell-templated><span data-role="display" ></span>'
                            } else if (column.data) {
                                cellTmpl += 'title="{{record.' + column.data + '}}"><span data-role="display">{{record.' + column.data + '}}</span>'
                            } else {
                                cellTmpl += '>'
                            }
                            if (column.editable || column.editorTmpl) {
                                cellTmpl += '<span data-role="editor"></span>';
                            }
                            return cellTmpl + '</td>';
                        })
                        .join('');
                    me.rowTmpl = '<tr{{if grid.rowClassController}} render-row-class outer-scope="grid.scope.$parent"{{/if}} {{if enableEvenClass === \'true\' && rowIndex % 2 === 1}} class="table-body-tr-even"{{/if}}>{{if grid.hasCheckbox}}<td class="grid-col-checkbox"><div class="form-clickbox{{if grid._isChecked(record)}} selected{{/if}}" render-checkbox-disabled outer-scope="grid.scope.$parent"><a href="javascript:void(0);" class="fi" ></a></div></td>{{/if}}' + columnsTmpl + '</tr>'
                }
                return me.rowTmpl;
            };


            GridProto._getCompiledRowTmpl = function () {
                var me = this;
                if (!me.compiledRowTmpl) {
                    me.compiledRowTmpl = artTmpl.compile(me._getRowTmpl());
                }
                return me.compiledRowTmpl
            };

            GridProto._getCompiledTableTmpl = function () {
                var me = this;
                if (!me.compiledTableTmpl) {
                    me.compiledTableTmpl = artTmpl.compile('{{each source as record rowIndex}}' + me._getRowTmpl() + '{{/each}}');
                }
                return me.compiledTableTmpl;
            };

            GridProto._registerHelpers = function () {
                var me = this,
                    columns;
                if (me.doneRegisterHelpers !== true) {
                    columns = me.columns;
                    _.chain(columns)
                        .map(_.property('data'))
                        .filter(_.negate(_.isEmpty))
                        .map(function (data) {
                            var firstVLPos = data.indexOf('|'),
                                filterString;
                            if (firstVLPos !== -1) {
                                filterString = data.substring(firstVLPos);
                                return filterString.match(/\|\s*[\$_\w\d]+/g)
                            }
                        })
                        .filter(_.negate(_.isEmpty))
                        .flatten()
                        .map(function (x) {
                            return $.trim(x.substring(1));
                        })
                        .each(function (helperName) {
                            artTmpl.helper(helperName, function () {
                                return $filter(helperName).apply($window, arguments);
                            });
                        });
                    me.doneRegisterHelpers = true;
                }
            };

            GridProto._syncRows = function (newSource, oldSource) {
                if (_.isUndefined(newSource)) return;
                var me = this,
                    scope = me.scope,
                    tbody = me.tbody,
                    trs = tbody.rows,
                    $rows = me.element.find('.table-body tr'),
                    added = Arrays.subtract(newSource, oldSource),
                    removed = Arrays.subtract(oldSource, newSource),
                    balanceCount = added.length - removed.length,
                    minRowIndex = $rows.length,
                    rowIndex, row, $newRow;
                angular.forEach(removed, function (r) {
                    rowIndex = Arrays.indexOf(oldSource, r);
                    row = trs[rowIndex];
                    row.parentNode.removeChild(row);
                    minRowIndex = Math.min(minRowIndex, rowIndex);
                });
                angular.forEach(added, function (r) {
                    rowIndex = Arrays.indexOf(newSource, r);
                    $newRow = angular.element(tbody.insertRow(rowIndex));
                    $newRow.replaceWith(me._getCompiledRowTmpl()({
                        record: r,
                        rowIndex: rowIndex,
                        grid: me
                    }));
                    $compile(trs[rowIndex])(scope);
                    minRowIndex = Math.min(minRowIndex, rowIndex);
                });
                me._keepLazyViewLength(balanceCount);
                me._refreshRows(minRowIndex);
            };

            GridProto._keepLazyViewLength = function (balanceCount) {
                var me = this,
                    scope = me.scope,
                    $renderedRows, renderedCount;
                if (scope.lazy === 'true') {
                    if (balanceCount > 0) {
                        $renderedRows = me.element.find('.grid-body tr');
                        renderedCount = $renderedRows.length;
                        $renderedRows.slice(renderedCount - balanceCount, renderedCount).remove();
                    } else {
                        var nextPage = me._getLazyNextPage(),
                            limit = nextPage * me._getLazyPageSize(),
                            rowDatas = scope.source.slice(limit + balanceCount, limit);
                        me._appendRowsAndStates(rowDatas);
                    }
                }
            };

            /**
             * 将指定<b>位置</b>或<b>数据对象</b>的行移动到指定位置， 默认移动到第一行
             *
             * @param rowDataOrIndex [Object/Number] 行数据或在`source`中的位置
             * @param [index=0] 移动到第几行， 默认移动到第一行
             */
            GridProto.moveRowTo = function (rowDataOrIndex, index) {
                var me = this,
                    scope = me.scope,
                    source = scope.source,
                    tbody = me.tbody,
                    trs = tbody.rows,
                    fromIndex = angular.isNumber(rowDataOrIndex) ? rowDataOrIndex : _.indexOf(source, rowDataOrIndex),
                    toIndex = angular.isNumber(index) ? index : 0,
                    rowData, fromTr, toTr;
                if (fromIndex >= 0 && toIndex >= 0) {
                    rowData = source[fromIndex];
                    source.splice(fromIndex, 1);
                    source.splice(toIndex, 0, rowData);

                    fromTr = trs[fromIndex];
                    toTr = trs[toIndex];
                    if (fromTr && toTr) {
                        tbody.insertBefore(trs[fromIndex], trs[toIndex]);
                        me._refreshRows(Math.min(fromIndex, toIndex), Math.max(fromIndex, toIndex) + 1);
                    } else if (scope.lazy === 'true' && (fromTr || toTr)) {
                        if (fromTr && !toTr) {
                            tbody.removeChild(fromTr);
                            me._keepLazyViewLength(-1);
                        } else {
                            tbody.insertRow(index);
                            $(trs[index]).replaceWith(me._getCompiledRowTmpl()({
                                rowIndex: fromIndex,
                                record: rowData,
                                grid: me,
                                enableEvenClass: scope.enableEvenClass
                            }));
                            $compile(trs[index])(scope);
                            me._keepLazyViewLength(1);
                        }
                        me._refreshRows(Math.min(fromIndex, toIndex), trs.length);
                    }
                }
            };

            GridProto._refreshRows = function (start, end) {
                var me = this,
                    scope = me.scope,
                    gridOuterScope = scope.$parent,
                    trs = Array.prototype.slice.apply(me.tbody.rows, arguments),
                    ending = end || scope.source.length,
                    range = _.range(start, ending);
                if (start >= 0 && start < ending) {
                    angular.forEach(trs, function (index, tr) {
                        angular.element(tr).toggleClass('table-body-tr-even', tr.rowIndex % 2 === 1)
                    });
                    gridOuterScope.$broadcast(CONSTANTS.REFRESH_ROWS, range);
                }
            };

            GridProto._getLazyPageSize = function () {
                var me = this;
                return _.result(me, '_lazyPageSize', function () {
                    var height = me.element.height();
                    return me._lazyPageSize = Math.ceil(height / 21);
                });
            };

            GridProto._getLazyNextPage = function () {
                if (_.isNumber(this._lazyPage)) {
                    return (this._lazyPage + 1);
                }
                return 0;
            };

            GridProto._getLazyPageData = function (page, pageSize) {
                var me = this,
                    source = me.scope.source,
                    start = page * pageSize,
                    end;
                if(source == null || source == "undefined"){
                    return [];
                }
                if (start >= source.length) {
                    return [];
                } else {
                    end = Math.min(source.length, start + pageSize);
                    return source.slice(start, end);
                }
            };

            GridProto._renderNextLazyPage = function () {
                var me = this,
                    page = me._getLazyNextPage(),
                    pageSize = me._getLazyPageSize(),
                    pageData = me._getLazyPageData(page, pageSize),
                    hasNextPage = !_.isEmpty(pageData);
                if (hasNextPage) {
                    me._lazyPage = page;
                    me._appendRowsAndStates(pageData)
                }
                return [];
            };

            GridProto._appendRowsAndStates = function (rowDatas) {
                var me = this,
                    scope = me.scope,
                    outerScope = scope.$parent,
                    $tbody = me.element.find('.grid-body tbody'),
                    firstRowIndex = _.indexOf(me.scope.source, rowDatas[0]),
                    rowIndexRange = _.range(firstRowIndex, firstRowIndex + me._getLazyPageSize()),
                    rowsSelector = firstRowIndex === 0 ? 'tr' : 'tr:gt(' + (firstRowIndex - 2) + ')',
                    $rows;
                $tbody.append(_.chain(rowDatas)
                    .map(function (record, index) {
                        return {
                            rowIndex: rowIndexRange[index],
                            record: record,
                            grid: me,
                            enableEvenClass: scope.enableEvenClass
                        };
                    })
                    .map(me._getCompiledRowTmpl())
                    .value()
                    .join(''));
                $rows = $tbody.children(rowsSelector);
                $compile($rows)(me.scope);
                if (!outerScope.$$phase && !outerScope.$root.$$phase) {
                    outerScope.$digest();
                }
            };

            GridProto._getColumnIndex = function (cellIndexOrDom) {
                var cellIndex;
                if (angular.isNumber(cellIndexOrDom)) {
                    cellIndex = cellIndexOrDom;
                } else {
                    cellIndex = angular.element(cellIndexOrDom)[0].cellIndex;
                }
                return this.hasCheckbox ? (cellIndex - 1) : cellIndex;
            };

            GridProto._render = function () {
                var me = this,
                    scope = me.scope,
                    oldTbody = me.element.find('table.table-body tbody'),
                    tbody;

                if (scope.lazy === 'true') {
                    oldTbody.replaceWith('<tbody></tbody>');
                    tbody = me.tbody = me.element.find('table.table-body tbody')[0];
                    delete me._lazyPage;
                    me._renderNextLazyPage();
                    me._renderNextLazyPage();
                } else {
                    oldTbody.replaceWith('<tbody></tbody>').data('$gDataGridController', me);
                    tbody = me.tbody = me.element.find('table.table-body tbody')[0];
                    tbody.innerHTML = me._getCompiledTableTmpl()(scope);
                    $compile(tbody)(scope);
                }
                me._reset();
            };

            GridProto._restColSettingsBtn = function () {
                var me = this,
                    scope = me.scope,
                    $headRow;
                if (!!scope.colSettingsKey) {
                    $headRow = me.element.find('div.grid-head tr:last');
                    $headRow.find('i[data-col-settings]').remove();
                    $headRow.find('th:visible:eq(' + (me.hasCheckbox ? 1 : 0) + ')').append($compile('<i data-col-settings class="iconfont2 fi-set set-btn" ng-click="grid._startSetting($event)"></i>')(scope));
                }
            };

            GridProto._enterCols = function (cols) {
                var me = this,
                    scope = me.scope;
                me._storeColumns(cols);
                me._reSortHeaders();
                me._restColSettingsBtn();
                me._restAllTmpl();
                me._registerHelpers();
                if (!(scope.$$phase || scope.$root.$$phase)) {
                    scope.$apply(function () {
                        me._render();
                    });
                } else {
                    me._render();
                }
            };

            GridProto._loadAndReSortHeadersIfStored = function () {
                var me = this;
                var colSettings = LocalStorages.get(me.scope.colSettingsKey);
                if (colSettings) {
                    me.__originalCols = me.columns;
                    me.columns = _.map(colSettings, function (colSetting) {
                        return me.__originalCols[colSetting.oIdx];
                    });
                    me._reSortHeaders();
                }
            };


            GridProto._getColSettings = function () {
                var me = this,
                    key = me.scope.colSettingsKey;
                if (key && !LocalStorages.get(key)){
                    me._storeColumns(me.columns);
                }
                return LocalStorages.get(key);
            };

            GridProto._storeColumns = function (cols) {
                var me = this;
                if (_.isUndefined(me.__originalCols)) {
                    me.__originalCols = me.columns;
                }
                me.columns = cols;
                LocalStorages.set(me.scope.colSettingsKey, _.map(cols, function (col) {
                    return {
                        oIdx: _.indexOf(me.__originalCols, col)
                    }
                }));
            };

            GridProto._restAllTmpl = function () {
                delete this.rowTmpl;
                delete this.compiledRowTmpl;
                delete this.compiledTableTmpl;
                delete this.doneRegisterHelpers;
            };

            GridProto._reSortHeaders = function () {
                var me = this,
                    $headRow = me.element.find('div.grid-head tr:last'),
                    headers = _.initial($headRow.children('th')),
                    tmpledCells = me.hasCheckbox ? _.rest(headers) : headers;
                _.chain(tmpledCells)
                    .map(angular.element)
                    .map(function ($cell) {
                        return {
                            $cell: $cell,
                            idx: _.indexOf(me.columns, getColumn($cell))
                        }
                    })
                    .filter(function (x) {
                        if (x.idx === -1) {
                            x.$cell.hide();
                            return false;
                        }
                        x.$cell.show();
                        return true;
                    })
                    .sortBy(_.property('idx'))
                    .map(_.property('$cell'))
                    .reduce(function ($left, $right) {
                        return $right.insertAfter($left);
                    });

                function getColumn($cell) {
                    return $cell.isolateScope().column;
                }
            };

            GridProto._startSetting = function ($event) {
                var me = this;
                ColSettings._startSetting(me);
                $event.stopPropagation();
            };

            GridProto.doLayout = function (autoSetWidthHeight) {
                var me = this,
                    document = $document[0],
                    documentElement = document.documentElement,
                    body = document.body,
                    bodyWidth = documentElement.clientWidth || body.clientWidth,
                    bodyHeight = documentElement.clientHeight || body.clientHeight,
                    attrs = me.attrs;
                if (!autoSetWidthHeight) {
                    if (attrs.gTop && attrs.gBottom) {
                        me.element.css("height", bodyHeight - attrs.gTop - attrs.gBottom);
                    }
                } else if (attrs.gBottom) {
                    me.element.css("height", bodyHeight - (me.element[0].getBoundingClientRect().top || attrs.gTop) - attrs.gBottom);
                }
                if (attrs.gLeft && attrs.gRight) {
                    me.element.css("width", bodyWidth - attrs.gLeft - attrs.gRight);
                }
            };


            return {
                template: '<div class="grid">\n    <div class="grid-head" ng-click="headClick($event)">\n        <table class="table-head">\n            <tbody>\n            <tr>\n                <th class="table-scroll-space"></th>\n            </tr>\n            </thead> </table>\n    </div>\n    <div class="grid-body" ng-click="bodyClick($event)" ng-dblclick="bodyDbClick($event)">\n        <table class="table-body"><tbody></tbody></table>    \n        <div class="col-resize-line"></div>\n    </div>\n</div> ',
                replace: true,
                restrict: 'E',
                transclude: true,
                scope: {
                    sourceName: '@sourceName',
                    validatorName: '@',
                    $columns: '=columns',
                    $selectedRow: '=selectedRow',
                    $checkedRows: '=checkedRows',
                    onLoadSuccess: '&',
                    onRender: '&',
                    onBeforeSortColumn: '&',
                    onSortColumn: '&',
                    onBeforeSelect: '&',
                    onSelect: '&',
                    canToggleSelected: '@',
                    enableEvenClass: '@',
                    onBeforeRowDbclick: '&',
                    onRowDbclick: '&',
                    onBeforeCellDbclick: '&',
                    onCellDbclick: '&',
                    lazy: '@',
                    colSettingsKey: '@',
                    gTop: '@',
                    gBottom: '@',
                    gLeft: '@',
                    gRight: '@'
                },
                require: ['gDataGrid', '?gItemClass'],
                compile: function (tElement, tAttributes, transclude) {
                    return function (scope, element, attrs, controllers) {
                        var grid = controllers[0],
                            $head = element.find('.grid-head'),
                            $gridBody = element.find('.grid-body'),
                            $headTableBody = $head.find('tbody'),
                            $table = $gridBody.children('table'),
                            $tbody = $table.children('tbody'),
                            isBindCheckedRows = attrs.hasOwnProperty('checkedRows'),
                            isBindSelectedRow = attrs.hasOwnProperty('selectedRow');

                        grid.rowClassController = controllers[1];
                        $tbody.data('$gDataGridController', grid);
                        if (scope.validatorName) {
                            grid.formName = 'grid' + grid.hash + 'form';
                            var $form = angular.element('<form>', {
                                name: grid.formName,
                                'g-validator': scope.validatorName,
                                'data-invalid-msg': 'tooltipMessenger',
                                'onsubmit': 'return false'
                            });
                            $table.wrap($form);
                            $form = $gridBody.children('form');
                            $compile($form)(scope);
                            grid.formController = $form.data('$formController');
                            grid.gValidatorController = $form.data('$gValidatorController');
                            $tbody.data('$formController', grid.formController);
                            $tbody.data('$gValidatorController', grid.gValidatorController);
                        }

                        transclude(scope, function (clone) {
                            var headCells = clone.filter('th'),
                                $headSpaceCol = $headTableBody.find('tr th.table-scroll-space'),
                                $foot = clone.filter('.grid-foot');
                            if (headCells.length > 0) {
                                angular.forEach(headCells, function (cell) {
                                    $headSpaceCol.before(cell);
                                });
                            }
                            if ($foot.length > 0) {
                                // init footData
                                grid.footCells = [];
                                element.append($foot);
                                // 绑定 scroll
                                var bl;
                                $gridBody.bind('scroll', function (event) {
                                    var element = (event.target || event.srcElement),
                                        nbl = element.scrollLeft;
                                    if (bl !== nbl) {
                                        $foot[0].scrollLeft = nbl;
                                        bl = nbl;
                                    }
                                });
                            }
                        }, grid);

                        grid._registerHeaderDraggable();
                        grid._loadAndReSortHeadersIfStored();
                        grid._renderCellsStyle();
                        grid._restColSettingsBtn();
                        grid.doLayout();

                        $dataSourceManager.getDataSource(attrs.sourceName).then(function (result) {
                            scope.dataSource = result;
                        });

                        scope.$on(scope.sourceName, function (event, result) {
                            grid.__rendered = true;
                            var body = $document[0].body;
                            if (body.hasAttribute('g-dict') && !!body.getAttribute('g-dict') && scope.$root.$dictReturned !== true) {
                                scope.$on('$dictReturned', function () {
                                    grid._registerHelpers();
                                    doLayer();
                                });
                            } else {
                                grid._registerHelpers();
                                doLayer();
                            }

                            function doLayer() {
                                grid.changeDataSourced = true; //为了标示不是外部改变了数据源
                                scope.source = result['records'];
                                if (result.hasOwnProperty('moreAttrs')) {
                                    scope.moreAttrs = result['moreAttrs'];
                                }

                                grid._render();

                                if (angular.isFunction(scope.onLoadSuccess)) {
                                    scope.onLoadSuccess({
                                        source: scope.source,
                                        grid: grid,
                                        result: result
                                    });
                                }
                            }
                        });

                        scope.headClick = function ($event) {
                            grid._headClick($event);
                        };

                        scope.bodyClick = function ($event) {
                            var target = $event.target || $event.srcElement,
                                $target = angular.element(target),
                                $cell = $target.closest('td'),
                                $row = $cell.closest('tr');
                            if ($target.is('a.fi') && $target.closest('td').is('.grid-col-checkbox')) {
                                grid._toggleCellChecked($event);
                            }
                            if ($row.length === 1) {
                                grid._rowClick($event, $row);
                            }
                        };

                        scope.bodyDbClick = function ($event) {
                            var target = $event.target || $event.srcElement,
                                $target = angular.element(target),
                                $cell = $target.closest('td'),
                                $row = $cell.closest('tr');
                            grid._rowDbClick($event, $row);
                            grid._cellDbClick($event, $cell);
                        };

                        // 实现 grid选中行双向绑定 start
                        scope.$watch('selectedRow', function (newRow, oldRow) {
                            if (newRow !== oldRow) {
                                if (isBindSelectedRow) {
                                    scope.$selectedRow = newRow;
                                }
                            }
                        });

                        if (isBindSelectedRow) {
                            scope.$watch('$selectedRow', function (newRow, oldRow) {
                                var rowIndex, $row;
                                if (newRow !== oldRow && scope.selectedRow !== newRow) {
                                    rowIndex = Arrays.indexOf(scope.source, newRow);
                                    $row = angular.element('tr:nth-child(' + rowIndex + ')', grid.tbody);
                                    grid._rowClick({}, $row);
                                }
                            });
                        }

                        if (isBindCheckedRows) {
                            if (angular.isArray(scope.$checkedRows)) {
                                scope.checkedRows = scope.$checkedRows;
                            } else {
                                scope.checkedRows = scope.$checkedRows = [];
                            }
                            scope.$watchCollection('$checkedRows', function () {
                                if (grid.checkRowByContrl !== true) {
                                    grid._syncCheckRowsByOuter();
                                }
                                grid.checkRowByContrl = false;
                            });
                        } else {
                            scope.checkedRows = [];
                        }

                        // 实现 grid选中行双向绑定 end

                        if (angular.isFunction(scope.onRender)) {
                            scope.onRender({
                                grid: grid,
                                source: scope.source
                            });
                        }

                        // 绑定 scroll
                        var bl;
                        $gridBody.bind('scroll', function (event) {
                            var element = (event.target || event.srcElement),
                                nbl = element.scrollLeft;
                            if (bl !== nbl) {
                                $head[0].scrollLeft = nbl;
                                bl = nbl;
                            }
                        });

                        $($window).on('resize', function () {
                            grid._renderCellsStyle();
                            grid.doLayout(true);
                        });

                        scope.$on('show', function () {
                            grid._renderCellsStyle();
                        });

                        scope.$watchCollection('source', function (newSource, oldSource) {
                            if (grid.changeDataSourced !== true) {
                                grid._syncRows(newSource, oldSource);
                            }
                            grid.changeDataSourced = false;
                        });

                        if (scope.lazy === 'true') {
                            element.children('.grid-body').on('scroll', function () {
                                var st = $(this).scrollTop(),
                                    loadedHeight = (grid._getLazyNextPage() - 2) * grid._getLazyPageSize() * 21;
                                if (st > loadedHeight) {
                                    grid._renderNextLazyPage();
                                }
                            });
                        }
                    };
                },
                controller: ['$scope', '$element', '$attrs', function ($scope, $element, $attrs) {
                    $scope.grid = new Grid($scope, $element, $attrs);
                    return $scope.grid;
                }]
            };
        })
        .directive('gCheckboxColumn', function () {
            return {
                restrict: 'E',
                template: '<th class="grid-col-checkbox"><g-checkbox ng-model="allChecked"></g-checkbox></th>',
                replace: true,
                scope: false,
                require: ['^gDataGrid', '?gItemDisabled'],
                link: function (scope, element, attrs, controllers) {
                    var gridController = controllers[0],
                        gItemDisabled = controllers[1];
                    gridController.hasCheckbox = true;
                    gridController.checkboxesDisabledController = gItemDisabled;
                    /** @namespace attrs.gItemDisabled */
                    gridController.checkboxesDisabledExpress = attrs.gItemDisabled;
                }
            };
        })
        .directive('gColumn', function () {

            function getRealField(rawData) {
                var firstFilterIdx;
                if (rawData) {
                    firstFilterIdx = rawData.indexOf('|');
                    if (firstFilterIdx !== -1) {
                        return $.trim(rawData.substring(0, firstFilterIdx));
                    }
                }
                return rawData;
            }

            function formatDataFilters(rawData) {
                if (!!rawData) {
                    return rawData.replace(/\s*\|\s*/g, ' | ');
                }
                return rawData;
            }

            function getCellAlignClass(scope) {
                return 'align-' + (scope.cellAlign || scope.align || 'left');
            }

            return {
                template: '<th>\n    <div class="grid-head-sort" ng-if="sortable !== undefined">\n        <button class="btn">\n            <span ng-class="{\'selected\': grid.sortName === sortable && grid.sortDirection === \'asc\'}" class="caret caret-up"></span>\n        </button>\n        <button class="btn">\n            <span ng-class="{\'selected\': grid.sortName === sortable && grid.sortDirection === \'desc\'}" class="caret caret-down"></span>\n        </button>\n    </div>\n</th>',
                replace: true,
                restrict: 'E',
                transclude: true,
                scope: {
                    $sortable: '@sortable',
                    data: '@',
                    width: '@',
                    editable: '@',
                    tmpl: '@',
                    disableTitle: '@',
                    align: '@',
                    cellAlign: '@'
                },
                require: ['^gDataGrid', '?gItemClass'],
                compile: function (tElement, tAttrs, transclude) {
                    return function (scope, element, attrs, controllers) {
                        var gridController = controllers[0],
                            gItemClass = controllers[1];
                        transclude(scope, function (clone) {
                            angular.forEach(clone, function (node) {
                                if (!/G-COLUMN-TMPL|G-COLUMN-EDITOR/i.test(node.tagName)) {
                                    element.append(node);
                                }
                            });
                        }, gridController);
                        var columnInstance = {
                                field: getRealField(scope.data),
                                data: formatDataFilters(scope.data),
                                text: $.trim(element.text()),
                                widthDef: scope.width,
                                editable: scope.editable,
                                tmpl: scope.tmpl,
                                element: element,
                                cellAlignClass: getCellAlignClass(scope),
                                gItemClass: gItemClass,
                                disableTitle: scope.disableTitle === 'true'
                            },
                            newLength = gridController._addColumn(columnInstance),
                            colIndex = newLength - 1,
                            cellIndex = element[0].cellIndex,
                            onBeforeCellDbclickDef = attrs.onBeforeCellDbclick,
                            onCellDbclickDef = attrs.onCellDbclick,
                            sortable;
                        // 如果需要排序， data中有空格视为包含过滤器， 截取空格前字符串作为排序字段
                        if (attrs.hasOwnProperty('sortable')) {
                            sortable = columnInstance.field;
                        }

                        if (!!scope.editorTmpl) {
                            columnInstance.editorTmpl = scope.editorTmpl;
                        }
                        if (columnInstance.field && (columnInstance.editable || columnInstance.editorTmpl)) {
                            columnInstance.canEdit = true;
                        }
                        // 列双击事件
                        if (onBeforeCellDbclickDef) {
                            columnInstance.onBeforeCellDbclick = gridController._getScopeEvent(onBeforeCellDbclickDef);
                        }
                        if (onCellDbclickDef) {
                            columnInstance.onCellDbclick = gridController._getScopeEvent(onCellDbclickDef);
                        }

                        columnInstance.colWidthClassName = gridController._getColWidthClassName(cellIndex);
                        columnInstance.index = colIndex;
                        columnInstance.sortable = scope.sortable = sortable;

                        scope.column = columnInstance;
                        scope.grid = gridController;
                        attrs.$addClass(columnInstance.colWidthClassName);

                    };
                }
            };
        })
        .directive('cellTemplated', function ($compile) {
            return {
                restrict: 'A',
                scope: true,
                require: '^gDataGrid',
                link: function (scope, element, attrs, grid) {
                    //noinspection JSUnresolvedVariable
                    var columnIndex = grid._getColumnIndex(element),
                        tr = element.parent()[0],
                        rowIndex = tr.rowIndex,
                        row = grid.scope.source[rowIndex],
                        column = grid.columns[columnIndex],
                        $displayPlace = element.children('[data-role=display]'),
                        tmplScope = grid._getCellTmplScope({
                            rowIndex: rowIndex,
                            column: column,
                            row: row,
                            columnIndex: columnIndex
                        });
                    $displayPlace.html(column.tmpl);
                    $compile($displayPlace)(tmplScope);
                    tmplScope.$on(CONSTANTS.REFRESH_ROWS, function (event, range) {
                        if (_.contains(range, tmplScope.rowIndex)) {
                            tmplScope.rowIndex = tr.rowIndex;
                        }
                    });
                }
            };
        })
        .directive('gFooter', function () {
            return {
                restrict: 'E',
                template: '<div class="grid-foot"> <table class="table-foot"> <tbody> <tr> <th class="table-scroll-space"></th> </tr> </tbody> </table> </div>',
                replace: true,
                transclude: true,
                scope: false,
                require: '^gDataGrid',
                compile: function (tElement, tAttrs, transclude) {
                    return function (scope, element, attrs, gridController) {
                        var $footSpaceCell = element.find('.table-scroll-space');
                        transclude(scope, function (clone) {
                            var footCells = clone.filter('th');
                            $footSpaceCell.before(footCells);
                        }, gridController);
                    }
                }
            };
        })
        .directive('gFooterCell', function () {
            return {
                restrict: 'E',
                template: '<th><div ng-transclude/> </th>',
                replace: true,
                transclude: true,
                scope: false,
                require: '^gDataGrid',
                link: function (scope, element, attrs, gridController) {
                    var span = Number(element.attr('colspan')),
                        spanColWidthClassName = gridController._registerFootCellWidthClass(span);

                    attrs.$addClass(spanColWidthClassName);
                }
            };
        })
        // just for stored column's template
        .directive('gColumnTmpl', function () {
            return {
                restrict: 'E',
                scope: false,
                compile: function (tElement) {
                    var columnTmpl = $.trim(tElement.html());
                    if (navigator.appName == "Microsoft Internet Explorer" && navigator.appVersion.split(";")[1].replace(/[ ]/g, "") == "MSIE8.0") {
                        columnTmpl = columnTmpl.replace('<:', '<').replace('<:/', '</');
                    }
                    return function (scope) {
                        scope.tmpl = columnTmpl;
                    }
                }
            };
        })
        // NOTE 一个表格中只能定义一个删除按钮
        .directive('gActionRemove', function (Arrays) {
            return {
                restrict: 'E',
                replace: 'true',
                template: '<a href="javascript:;" ng-click="removeRecord($event)" class="btn" title="删除"><i class="fi fi-del"></i></a>',
                scope: false,
                link: function (scope, element, attrs) {
                    var grid = scope.grid,
                        onBeforeActionAttrVal = attrs['onBeforeAction'],
                        onActionAttrVal = attrs['onAction'],
                        onBeforeAction = angular.noop,
                        onAction = angular.noop;
                    if (grid) {
                        if (!!onBeforeActionAttrVal) {
                            onBeforeAction = grid._getScopeEvent(onBeforeActionAttrVal);
                        }
                        if (!!onActionAttrVal) {
                            onAction = grid._getScopeEvent(onActionAttrVal);
                        }
                    }

                    scope.removeRecord = function ($event) {
                        var element = $event.target || $event.srcElement,
                            $row = angular.element(element).closest('tr'),
                            rowIndex = $row[0].rowIndex,
                            source = grid.scope.source,
                            row = source[rowIndex];
                        //noinspection JSUnresolvedVariable
                        var eventParams = {
                            $event: $event,
                            record: row
                        };
                        if (onBeforeAction(eventParams) !== false) {
                            Arrays.remove(grid.scope.checkedRows, source[rowIndex]);
                            source.splice(rowIndex, 1);
                            onAction(eventParams);
                        }
                    }
                }
            };
        })
        .directive('gColumnEditor', function () {
            return {
                restrict: 'E',
                scope: false,
                compile: function (tElement) {
                    var editorTmpl = $.trim(tElement.html());
                    if (navigator.appName == "Microsoft Internet Explorer" && navigator.appVersion.split(";")[1].replace(/[ ]/g, "") == "MSIE8.0") {
                        editorTmpl = editorTmpl.replace('<:', '<').replace('<:/', '</');
                    }
                    return function (scope) {
                        scope.editorTmpl = editorTmpl;
                    }
                }
            };
        })
        .directive('renderCheckboxDisabled', function () {
            return {
                require: 'outerScope',
                scope: true,
                link: function (scope, element, attrs, outerScopeController) {
                    var gItemDisabledController = scope.grid.checkboxesDisabledController,
                        locals, rowIndex, tr;
                    if (angular.isFunction(gItemDisabledController)) {
                        tr = element.closest('tr')[0];
                        rowIndex = tr.rowIndex;
                        locals = {
                            rowIndex: rowIndex,
                            row: scope.source[rowIndex]
                        };
                        angular.extend(scope, locals);
                        gItemDisabledController({
                            locals: locals,
                            scope: scope,
                            element: element,
                            outerScopeController: outerScopeController
                        });

                        scope.$on(CONSTANTS.REFRESH_ROWS, function (event, range) {
                            rowIndex = tr.rowIndex;
                            if (_.contains(range, rowIndex)) {
                                scope.rowIndex = rowIndex;
                            }
                        });
                    }
                }
            };
        })
        .directive('renderRowClass', function () {
            return {
                require: 'outerScope',
                scope: true,
                link: function (scope, element, attrs, outerScopeController) {
                    var rowClassController = scope.grid.rowClassController,
                        locals, rowIndex, tr;
                    if (angular.isFunction(rowClassController)) {
                        tr = element.closest('tr')[0];
                        rowIndex = tr.rowIndex;
                        locals = {
                            rowIndex: rowIndex,
                            row: scope.source[rowIndex]
                        };
                        angular.extend(scope, locals);
                        rowClassController({
                            locals: locals,
                            scope: scope,
                            element: element,
                            attrs: attrs,
                            outerScopeController: outerScopeController
                        });

                        scope.$on(CONSTANTS.REFRESH_ROWS, function (event, range) {
                            rowIndex = tr.rowIndex;
                            if (_.contains(range, rowIndex)) {
                                scope.rowIndex = rowIndex;
                            }
                        });
                    }
                }
            };
        })
        .directive('renderCellClass', function () {
            return {
                require: 'outerScope',
                scope: true,
                link: function (scope, element, attrs, outerScopeController) {
                    var cell = element.closest('td')[0],
                        grid = scope.grid,
                        columnIndex = grid._getColumnIndex(cell),
                        column = scope.grid.columns[columnIndex],
                        gItemClass = column.gItemClass,
                        locals, rowIndex, tr;
                    if (column && angular.isFunction(gItemClass)) {
                        tr = cell.parentNode;
                        rowIndex = tr.rowIndex;
                        locals = {
                            columnIndex: columnIndex,
                            column: column,
                            rowIndex: rowIndex,
                            row: scope.source[rowIndex]
                        };
                        angular.extend(scope, locals);
                        gItemClass({
                            locals: locals,
                            scope: scope,
                            attrs: attrs,
                            element: element,
                            outerScopeController: outerScopeController
                        });

                        scope.$on(CONSTANTS.REFRESH_ROWS, function (event, range) {
                            rowIndex = tr.rowIndex;
                            if (_.contains(range, rowIndex)) {
                                scope.rowIndex = rowIndex;
                            }
                        });
                    }
                }
            };
        });
});