﻿define('framework/directive/ControlValidatorServiceConstructor', [
        'angular',
        'jquery',
        'framework/directive/ControlValidator',
        '/static/config.properties.js'
    ], function (angular, $, ControlValidator, config) {
        var ctx = config.$paths.$current.ctx;
        return function (Arrays, Predicates, $http, $timeout, IllegalMessages) {
            var prototypeRef = ControlValidator.prototype;

            prototypeRef.fillListenControls = function () {
                var me = this,
                    listenProperties = me.getListenProperties(),
                    listenControls = me.listenControls = [];

                if (me.isCombineRule()) {
                    angular.forEach(listenProperties, function (prop) {
                        Arrays.pushAll(me.getCombineMatchedControls(prop), listenControls);
                    });
                } else {
                    me.listenControls = Arrays.transform(listenProperties, function (prop) {
                        return me.form[prop];
                    });
                }
                return me;
            };

            prototypeRef.fillListenControlsTargets$ = function () {
                var me = this;
                angular.forEach(me.listenControls, function (listenControl) {
                    if (listenControl) {
                        me.fillListenControlTargets$(listenControl);
                    }
                });
                return me;
            };

            /**
             * @private
             */
            prototypeRef.fillListenControlTargets$ = function (listenControl) {
                var me = this,
                    $form = me.$form,
                    controlName = listenControl.$name,
                    rule = me.rule,
                    $listenPropTargetCache = me.$listenPropTargetCache,
                    $outerContainerCache = me.$outerContainerCache,
                    $verifyTargetCache = me.$verifyTargetCache,
                    msgControl, $msgTarget;
                if (me.isOverrideListens() && rule.property) {
                    msgControl = me.getSameCombineControl(listenControl, rule.property);
                    $msgTarget = $listenPropTargetCache[controlName] = $form.find('[name=\'' + msgControl.$name + '\']')
                } else {
                    $msgTarget = $listenPropTargetCache[controlName] = $form.find('[name=\'' + controlName + '\']')
                }
                // 默认与 $target 一致
                $outerContainerCache[controlName] = $verifyTargetCache[controlName] = $msgTarget;
                if ($msgTarget.is('[outer-container]')) {
                    $outerContainerCache[controlName] = $msgTarget.parent().closest('[outer-container]');
                }
                if ($msgTarget.is('[verify-target]')) {
                    $verifyTargetCache[controlName] = $msgTarget.find('[verify-target]');
                }
            };

            prototypeRef.getListenProperties = function () {
                var me = this,
                    rule = me.rule;
                if (me.isOverrideListens()) {
                    return rule.listenProperties;
                } else if (rule.property) {
                    return [rule.property];
                } else if (rule.properties) {
                    return rule.properties;
                }
                return [];
            };

            prototypeRef.isOverrideListens = function () {
                return this.rule.listenProperties;
            };

            prototypeRef.getMsgTargetProperties = function () {
                var rule = this.rule;
                if (rule.property) {
                    return [rule.property];
                } else if (rule.properties) {
                    return rule.properties;
                }
                return [];
             };

            prototypeRef.isCombineRule = function(){
                return this.rule.combineName;
            };

            prototypeRef.getCombineMatchedControls = function (prop) {
                var me = this,
                    rule = me.rule,
                    combineName = rule.combineName,
                    form = me.form,
                    matchedControls = [],
                    reg;
                if (prop) {
                    reg = new RegExp('^' + combineName + '(\\\[\\\d+\\\])?\.' + prop + '$');
                    angular.forEach(form, function (control, fieldName) {
                        if (!(fieldName === 'verify' || /^\$/.test(fieldName)) && reg.test(fieldName)) {
                            matchedControls.push(control);
                        }
                    })
                }
                return matchedControls;
            };

            prototypeRef.apply = function () {
                var me = this,
                    gValidator = me.gValidator,
                    listenControls = me.listenControls,
                    rule = me.rule,
                    ruleName = me.ruleName;
                if (Arrays.isNotEmptyArray(listenControls)) {
                    angular.forEach(listenControls, function (modelControl) {
                        if (modelControl) {
                            var rulePredicate = me[ruleName],
                                $name = modelControl.$name;

                            function verifyFn(modelValue) {
                                var isValid;
                                if (angular.isFunction(rulePredicate) && gValidator.disabled !== true) {
                                    isValid = rulePredicate.call(me, modelValue, modelControl);
                                    if (rule.async !== true) {
                                        me.setValidityAndSendMessage(isValid, modelControl);
                                    }
                                }
                                return modelValue;
                            }

                            modelControl.$parsers.unshift(verifyFn);
                            modelControl.$formatters.unshift(verifyFn);
                            if (!angular.isFunction(modelControl.verify)) {
                                modelControl.verify = function () {
                                    angular.forEach(modelControl.$parsers, function (parseFn) {
                                        parseFn(modelControl.$viewValue);
                                    });
                                };
                                me.$verifyTargetCache[$name].bind('blur', modelControl.verify);
                            }
                        }
                    });
                }
                return me;
            };

            prototypeRef.setValidityAndSendMessage = function (isValid, modelControl){
                var me = this,
                    ruleName = me.ruleName,
                    rule = me.rule,
                    listenProperties, sameCombineListenControls,
                    mp;
                if (rule.property) {
                    modelControl.$setValidity(ruleName, isValid);
                    mp = me.getMessageParams(isValid, modelControl);
                    IllegalMessages.send(mp);
                } else {
                    listenProperties = me.getListenProperties();
                    sameCombineListenControls = me.getSameCombineControls(modelControl, listenProperties);
                    angular.forEach(sameCombineListenControls, function (control) {
                        control.$setValidity(ruleName, isValid);
                        mp = me.getMessageParams(isValid, control);
                        IllegalMessages.send(mp);
                    });
                }
            };

            /**
             * @deprecated
             */
            prototypeRef.sendIllegalMessages = function (isValid, modelControl) {
                var me = this,
                    rule = me.rule,
                    listenProperties, sameCombineListenControls,
                    mp;
                if (rule.property) {
                    mp = me.getMessageParams(isValid, modelControl);
                    IllegalMessages.send(mp);
                } else {
                    listenProperties = me.getListenProperties();
                    sameCombineListenControls = me.getSameCombineControls(modelControl, listenProperties);
                    angular.forEach(sameCombineListenControls, function (control) {
                        mp = me.getMessageParams(isValid, control);
                        IllegalMessages.send(mp);
                    });
                }
            };

            prototypeRef.getMessageParams = function (isValid, modelController) {
                var me = this,
                    formName = me.form ? me.form.$name : '',
                    rule = me.rule,
                    ruleName = rule.ruleName,
                    messageKey = rule.message,
                    gValidator = me.gValidator,
                    entityI18n = gValidator.entityI18n,
                    $outerContainer = me.$outerContainerCache[modelController.$name],
                    $verifyTarget = me.$verifyTargetCache[modelController.$name];
                return {
                    isValid: isValid,
                    $outerContainer: $outerContainer,
                    $verifyTarget: $verifyTarget,
                    message: entityI18n[messageKey] || '',
                    ruleName: ruleName,
                    formName: formName,
                    messengerName: $outerContainer.data('invalid-msg') || gValidator.defaultIllegalTarget
                };
            };

            prototypeRef.clearMessages = function () {
                var defaultMessengerName = this.gValidator.defaultIllegalTarget;

                angular.forEach(this.$outerContainerCache, function ($target) {
                    var messengerName = $target.data('invalid-msg') || defaultMessengerName;
                    IllegalMessages.clear(messengerName, $target);
                });
            };

            /**
             * 设置当前规则正在监听的控件的 angular 效验状态
             * @deprecated
             */
            prototypeRef.setListenControlsValidity = function (isValid, modelControl) {
                var me = this,
                    ruleName = me.ruleName,
                    rule = me.rule,
                    listenProperties, sameCombineListenControls;
                if (rule.property) {
                    modelControl.$setValidity(ruleName, isValid);
                } else {
                    listenProperties = me.getListenProperties();
                    sameCombineListenControls = me.getSameCombineControls(modelControl, listenProperties);
                    angular.forEach(sameCombineListenControls, function (control) {
                        control.$setValidity(ruleName, isValid);
                    });
                }
            };

            prototypeRef.notNull = function (rawValue) {
                return angular.isDefined(rawValue) && rawValue !== null;
            };

            prototypeRef.notEmpty = function (rawValue) {
                if (this.notNull(rawValue)) {
                    if (angular.isString(rawValue) || angular.isArray(rawValue)) {
                        return rawValue.length > 0;
                    } else {
                        throw new TypeError('NotEmpty 只支持 String 或 Array 类型的值的验证');
                    }
                }
                return false;
            };

            prototypeRef.notBlank = function (rawValue) {
                return !rawValue || (angular.isString(rawValue) && $.trim(rawValue).length > 0);
            };

            prototypeRef.require = function (rawValue) {
                if (angular.isString(rawValue)) {
                    return rawValue.length > 0;
                } else {
                    return this.notNull(rawValue);
                }
            };
            prototypeRef.numberRange = function(rawValue){
                return this.range(rawValue);
            };
            prototypeRef.pattern = function (rawValue) {
                var rule = this.rule;

                return !rawValue || (getReg(rule.pattern, rule.flags).test(rawValue));

                function getReg(pattern, flags) {
                    if (flags) {
                        return new RegExp(pattern, flags);
                    } else {
                        return new RegExp(pattern);
                    }
                }
            };

            prototypeRef.min = function (rawValue) {
                var value = Number(rawValue);
                return this.isEmptyString(rawValue) || (angular.isNumber(value) && value >= this.rule.min);
            };

            prototypeRef.max = function (rawValue) {
                var value = Number(rawValue);
                return this.isEmptyString(rawValue) || (angular.isNumber(value) && value <= this.rule.max);
            };

            prototypeRef.length = function (rawValue) {
                return angular.isUndefined(rawValue) || rawValue == null || (angular.isString(rawValue) && rawValue.length <= this.rule.max && rawValue.length >= this.rule.min);
            };

            prototypeRef.numeric = function (rawValue) {
                var value, token;
                if (rawValue) {
                    value = Number(rawValue);
                    token = rawValue.split('.');

                    if (this.rule.precision >= 0 && (token[1] || '').length > this.rule.precision) {
                        return false;
                    } else if (this.rule.sign === 'POSITIVE' && value < 0) {
                        return false;
                    } else if (this.rule.sign === 'NEGATIVE' && value > 0) {
                        return false;
                    }
                }
                return true;
            };

            prototypeRef.range = function (rawValue) {
                var value = Number(rawValue);
                return this.isEmptyString(rawValue) || (angular.isNumber(value) && value >= this.rule.min && value <= this.rule.max);
            };

            prototypeRef.email = function (rawValue) {
                if (!rawValue) {
                    return true;
                }

                var emailParts = rawValue.toString().split("@");
                if (emailParts.length != 2) {
                    return false;
                }

                if (/\.$/.test(emailParts[0]) || /\.$/.test(emailParts[1])) {
                    return false;
                }

                if (!/[a-z0-9!#$%&'*+/=?^_`{|}~-]+(\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*/i.test(emailParts[0])) {
                    return false;
                }

                return /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)+|\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\]/i.test(emailParts[1]);
            };

            /**
             *
             * @param rawValue {Date}
             */
            prototypeRef.dateMax = function (rawValue) {
                if (rawValue == null || !angular.isDate(rawValue)) return true;
                var me = this,
                    rule = me.rule,
                    compareTime = rule.compareTime || new Date().getTime(),
                    shiftedTime = rule.shiftedTime;
                return (compareTime + shiftedTime) >= rawValue.getTime();
            };

            /**
             *
             * @param rawValue {Date}
             */
            prototypeRef.dateMin = function (rawValue) {
                if (rawValue == null || !angular.isDate(rawValue)) return true;
                var me = this,
                    rule = me.rule,
                    compareTime = rule.compareTime || new Date().getTime(),
                    shiftedTime = rule.shiftedTime;
                return (compareTime + shiftedTime) <= rawValue.getTime();
            };

            prototypeRef.allMatch = function (rawValue, modelControl) {
                var me = this,
                    valMap = {},
                    allMatchControls = me.getSameCombineControls(modelControl, me.rule.listenProperties).concat(modelControl),
                    i = 0, modelValue;
                angular.forEach(allMatchControls, function (control) {
                    valMap[control.$viewValue] = undefined;
                });
                for (modelValue in valMap) {
                    if (valMap.hasOwnProperty(modelValue) && modelValue) {
                        i++;
                    }
                }
                return i <= 1;
            };

            prototypeRef.compare = function (rawValue, modelControl) {
                var me = this,
                    rule = me.rule,
                    than = rule.than,
                    // compare 需要值的具体类型， 不适合使用 $viewValue
                    propertyValue = modelControl.$modelValue,
                    compareToVal = me.getSameCombineControl(modelControl, rule.compareTo).$modelValue;
                if (!(propertyValue && compareToVal)) return true;
                if (!angular.isDate(propertyValue) && !angular.isDate(compareToVal)) {
                    propertyValue = Number.isNaN(Number(propertyValue))?propertyValue:Number(propertyValue);
                    compareToVal = Number.isNaN(Number(compareToVal))?compareToVal:Number(compareToVal);
                }
                switch (than) {
                    case 'GREATER':
                        return propertyValue > compareToVal;
                    case 'GREATER_EQ':
                        return propertyValue >= compareToVal;
                    case 'LESS':
                        return propertyValue < compareToVal;
                    case 'LESS_EQ':
                        return propertyValue <= compareToVal;
                    default:
                        return true;
                }
            };

            prototypeRef.unique = function (value, modelControl) {
                var me = this,
                    rule = me.rule,
                    entity = rule.entity,
                    pkProp = rule.pkProperty,
                    properties = rule.properties,
                    existsControls = me.getSameCombineControls(modelControl, properties),
                    fieldNames = [], fieldValues = [],
                    pkControl,
                    pkValue = '',
                    validCache = me.__validCache = me.__validCache || {};
                angular.forEach(existsControls, function (v) {
                    fieldNames.push(v.$name);
                    fieldValues.push((angular.isDefined(v.$viewValue)&&v.$viewValue!=null)?v.$viewValue:'');
                });
                pkControl = me.getSameCombineControl(modelControl, pkProp);
                if (pkControl && angular.isDefined(pkControl.$viewValue) && pkControl.$viewValue!=null) {
                    pkValue = pkControl.$viewValue;
                }

                if (value) {
                    if (me.reValidTimer) {
                        $timeout.cancel(me.reValidTimer);
                        me.reValidTimer = undefined;
                    }
                    me.reValidTimer = $timeout(function () {
                        var jsonKey =  JSON.stringify({
                            entityName: entity,
                            pkValue: pkValue,
                            fieldNames: fieldNames,
                            fieldValues: fieldValues
                        });
                        if (angular.isUndefined(validCache[jsonKey])) {
                            $http.post(ctx + '/validation/unique', jsonKey, {failHandler: false}).success(function (result) {
                                var isValid = result && result.success === true;
                                me.setValidityAndSendMessage(isValid, modelControl);
                                me.reValidTimer = undefined;
                                validCache[jsonKey] = isValid;
                            });
                        } else {
                            me.setValidityAndSendMessage(validCache[jsonKey], modelControl);
                            me.reValidTimer = undefined;
                        }
                    }, 300);
                } else {
                    me.setValidityAndSendMessage(true, modelControl)
                }
            };

            prototypeRef.eitherNotBlank = function (value, modelControl) {
                var me = this,
                    properties = me.properties,
                    controls = me.getSameCombineControls(modelControl, properties);
                return Arrays.exists(controls, Predicates.newPropPredicate('$viewValue', function (val) {
                    return angular.isString(val) && $.trim(val).length > 0;
                }));
            };

            prototypeRef.dateSpan = function (rawValue, modelControl) {
                var me = this,
                    rule = me.rule,
                    limit = rule.limit,
                    minDateField = me.getSameCombineControl(modelControl, rule.minDateField),
                    maxDateField = me.getSameCombineControl(modelControl, rule.maxDateField),
                    minDate, maxDate, span;
                if (minDateField && maxDateField) {
                    minDate = minDateField.$viewValue;
                    maxDate = maxDateField.$viewValue;
                    if (minDate && maxDate) {
                        span = maxDate.getTime() - minDate.getTime();
                        return span >= 0 && span < limit;
                    }
                }
                return true;
            };

            prototypeRef.isEmptyString = function (value) {
                return !this.notNull(value) || (angular.isString(value) && value.length == 0);
            };

            prototypeRef.getSameCombineControls = function (modelControl, properties) {
                var me = this,
                    sameCombineControls = [];
                angular.forEach(properties, function (prop) {
                    if (prop) {
                        var sameCombineControl = me.getSameCombineControl(modelControl, prop);
                        if (sameCombineControl) {
                            sameCombineControls.push(sameCombineControl)
                        }
                    }
                });
                return sameCombineControls;
            };

            prototypeRef.getSameCombineControl = function (modelControl, property) {
                var me = this,
                    rule = me.rule,
                    form = me.form,
                    controlName = modelControl.$name;
                if (rule.combineName) {
                    var matched = controlName.match(/\[\d]/);
                    if (Arrays.isNotEmptyArray(matched)) {
                        return form[rule.combineName + matched[0] + '.' + property];
                    }
                } else {
                    return form[property];
                }
            };

            return {
                /**
                 *
                 * @param gValidator {FormValidatorController}
                 * @return 控件效验器
                 */
                from: function (gValidator) {
                    return new ControlValidator(gValidator);
                },

                isSupported: function (ruleName) {
                    return angular.isFunction(ControlValidator.prototype[ruleName]);
                }
            }
        };
    }
)
;