/**
 * Created with IntelliJ IDEA.
 * User: zhengry
 * Date: 2014/11/18
 * Time: 15:54
 */
define([
    'require',
    'angular',
    '/static/config.properties.js'
], function (require, angular, config) {
    var currentPath = config.$paths.$current;
    return function ($http, $window, $compile, I18NLoader, ControlValidatorService, Arrays, IllegalMessages) {

        var gValidatorProto = FormValidatorController.prototype;

        /**
         * @cfg params.$form {jQuery()} 表单元素
         * @cfg params.entityName {String} 实体名称
         * @cfg params.defaultIllegalTarget {String} 默认错误消息输出位置
         * @constructor
         */
        function FormValidatorController(params) {
            angular.extend(this, params);
            this.controlValidators = [];
        }

        gValidatorProto.setRules = function (rules) {
            this.rules = rules;
        };

        gValidatorProto.getFieldRules = function (fieldName) {
            var lastDotIndex = fieldName.indexOf('.'),
                inCombine = lastDotIndex !== -1,
                propertyName = fieldName;
            if (inCombine) {
                propertyName = fieldName.substr(lastDotIndex + 1);
            }
            return Arrays.filter(this.rules, function (rule) {
                if (inCombine && !(new RegExp('^' + rule.combineName + '(\\\[\\\d+\\\])?\.' + propertyName + '$')).test(fieldName)) {
                    return false;
                }
                if (rule.listenProperties) {
                    return Arrays.exists(rule.listenProperties, propertyName);
                } else if (rule.property) {
                    return rule.property === propertyName;
                } else if (rule.properties) {
                    return Arrays.exists(rule.properties, propertyName);
                }
                return false;
            });
        };

        gValidatorProto.setEntityI18n = function (entityI18n) {
            this.entityI18n = entityI18n;
        };

        gValidatorProto.setForm = function (form) {
            this.form = form;
        };

        gValidatorProto.setForm$ = function ($form) {
            this.$form = $form;
        };

        gValidatorProto.forEachFieldValidator = function () {
            var me = this;
            angular.forEach(me.form, function (field, fieldName) {
                if (/^[^\$]/.test(fieldName) && !angular.isFunction(field)) {
                    me.getFieldRules(fieldName);
                }
            });
        };

        //noinspection JSUnusedLocalSymbols
        /**
         *
         * @cfg params.gValidator {FormValidatorController} 表单校验器
         * @cfg params.field {ngModelController} 字段
         * @cfg params.rules {[Object]} 校验规则
         * @constructor
         */
        function FieldValidator(params) {

        }


        gValidatorProto.forEachRulesValidator = function (scope, rules, control) {
            var me = this;
            angular.forEach(rules, function (rule) {
                new RuleValidator({
                    rule: rule,
                    gValidator: me,
                    control: control
                });
            });
        };

        gValidatorProto.disable = function () {
            this.disabled = true;
        };

        gValidatorProto.enable = function () {
            this.disabled = false;
        };

        gValidatorProto.reset = function () {
            var me = this,
                controlValidators = me.controlValidators;
            angular.forEach(controlValidators, function (controlValidator) {

                controlValidator.clearMessages();
                angular.forEach(controlValidator.listenControls, function (modelController) {
                    if (modelController) {
                        modelController.$setValidity(controlValidator.rule.ruleName, true);
                    }
                });

            })
        };

        /**
         *
         * @param params.rule {[Object]} 当前字段的校验规则
         * @param params.gValidator {FormValidatorController}
         * @param params.control {ngModelController}
         * @constructor
         */
        function RuleValidator(params) {
            var me = this;
            me.rule = params.rule;
            me.gValidator = params.gValidator;
            me.applyValidator(params.control);
        }

        RuleValidator.prototype.applyValidator = function (control) {
            var me = this,
                rule = me.rule,
                controlValidators = me.gValidator.controlValidators,
                listenProperties;
            if (ControlValidatorService.isSupported(rule.ruleName)) {
                var controlValidator = ControlValidatorService.from(me);
                if (control) {
                    // fillListenControls start
                    listenProperties = controlValidator.getListenProperties();
                    if (listenProperties.length === 1) {
                        controlValidator.listenControls = [control];
                    } else {
                        controlValidator.listenControls = controlValidator.getSameCombineControls(listenProperties);
                    }
                    // fillListenControls end
                    controlValidator
                        .fillListenControlsTargets$()
                        .apply();
                } else {
                    controlValidator
                        .fillListenControls()
                        .fillListenControlsTargets$()
                        .apply();
                }
                controlValidators.push(controlValidator);
            }
        };

        return {
            restrict: 'A',
            require: ['form', 'gValidator'],
            scope: false,
            link: function (scope, element, attrs, controllers) {
                var form = controllers[0],
                    gValidator = controllers[1],
                    entityAndGroupName = attrs['gValidator'],
                    outerValidatorName = attrs.validationApi;

                if (!!outerValidatorName) {
                    scope[outerValidatorName] = gValidator;
                }

                if (!entityAndGroupName) {
                    throw new ReferenceError("require entityAndGroupName");
                }
                require([currentPath.path + '/static/app/validation/generated/' + entityAndGroupName + ".js",
                    'i18n!' + currentPath.path + '/static/app/nls/entity-' + entityAndGroupName + ".js"], function (validationData, entityI18n) {
                    var rules = validationData.rules;
                    gValidator.setForm(form);
                    gValidator.setRules(rules);
                    gValidator.setEntityI18n(entityI18n);

                    gValidator.forEachRulesValidator(scope, rules);

                    form.verify = function () {
                        var formName = form.$name;
                        IllegalMessages.startValidForm(formName);
                        angular.forEach(form, function (field, fieldName) {
                            if (/^[^\$]/.test(fieldName) && angular.isFunction(field.verify)) {
                                field.verify();
                            }
                        });
                        if (!(scope.$$phase || scope.$root.$$phase)) {
                            scope.$digest();
                        }
                    };
                    if (!(scope.$$phase || scope.$root.$$phase)) {
                        scope.$digest();
                    }
                    form.$validator = gValidator;
                });
            },
            controller: ['$scope', '$element', '$attrs', function ($scope, $element, $attrs) {
                var entityAndGroupName = $attrs['gValidator'],
                    entityName = entityAndGroupName.split('_')[0],
                    defaultIllegalTarget = $attrs['invalidMsg'];
                return new FormValidatorController({
                    $form: $element,
                    entityName: entityName,
                    defaultIllegalTarget: defaultIllegalTarget
                });
            }]
        }
    };
});