define('framework/print/PrintSettingsAModule', [
    'angular',
    'underscore',
    'artTmpl'
], function (angular, _, artTmpl) {

    function getMainWindow(window) {
        var href = window.location.href;
        if (href.indexOf('__showUrl=true') !== -1) {
            return getMainWindow(window.parent);
        }
        return window;
    }

    var headerTmpl = artTmpl.compile('<tr>{{each selected as column}}<td{{if column.width}} width="{{column.width}}"{{/if}}>{{column.text}}</td>{{/each}}</tr>'),
        dataRowTmpl = artTmpl.compile('<tr>{{each selected as column}}<td{{if column.width}} width="{{column.width}}"{{/if}}>{{column.field}}</td>{{/each}}</tr>');

    return angular.module('PrintSettingsAModule', [])
        .directive('gPrintSettings', function ($parse, $rootScope, $window, GillionMsg, GillionLocationService, Arrays) {
            var PrintSettingsProto = PrintSettings.prototype;

            function PrintSettings(scope, element) {
                var me = this;
                me.scope = scope;
                me.element = element;
                me.selections = {
                    L: [],
                    R: []
                };
            }

            /**
             * 开始设置打印模板
             *
             * @param params.originalColumns {Array<Object>} 原始列信息
             * @param params.oldTemplate {String} 上次保存的模板
             * @param params.callback {function(string)} 确认设置模板后回调
             */
            PrintSettingsProto.startEdit = function (params) {
                var me = this,
                    oldCols;
                me.sededCallback = params.callback;
                me.source = params.originalColumns;
                ///me.printTmpl = params.printTmpl;
                me.oldTemplate = params.oldTemplate;
                me.parsePrintTmplCall = params.parsePrintTmplCall || function(){};
                if (params.oldTemplate) {
                    oldCols = me.parsePrintTmpl(params.oldTemplate, params.headerReg, params.dataReg)
                    me.selected = extractSelected(me.source, oldCols);
                } else {
                    me.selected = [];
                    Arrays.pushAll(me.source, me.selected);
                }
                me.show();
            };

            function extractSelected(source, oldCols) {
                var selected = [];
                _.each(source, function (col) {
                    var existsOldCol = _.findWhere(oldCols, {field: col.field});
                    if (existsOldCol) {
                        col.width = existsOldCol.width;
                        selected.push(col);
                    }
                });
                return selected;
            }

            PrintSettingsProto.parsePrintTmpl = function (printTemplate, headerReg, dataReg) {
                if(!printTemplate || !headerReg || !dataReg){
                    throw new Error('解析打印模板正则表达式不合法');
                }

                var headerFlag = printTemplate.match(headerReg || /LoopStar(.*)<\/tr>LoopContent/m),
                    headerRow = headerFlag[1],
                    dataFlag = printTemplate.match(dataReg || /\$TABLE_E\$\$TR_S\$(.*)\$TR_E\$/m),
                    dataRow = dataFlag[1],
                    $header, $dataRow, datas, columns, i, len, col;
                if (!headerRow) {
                    throw new Error('不能解析到打印列表头部， 请检查传入的打印模板是否合法 :' + printTemplate);
                }
                if (!dataRow) {
                    throw new Error('不能解析到打印列表数据行， 请检查传入的打印模板是否合法 :' + printTemplate);
                }
                $header = angular.element(headerRow);
                columns = _.chain($header.children('td'))
                    .map(angular.element)
                    .map(function ($cell) {
                        var col = {text: $cell.text()},
                            widthStr = $cell.attr('width'),
                            width = widthStr ? Number(widthStr) : undefined;
                        if (!isNaN(width) && _.isNumber(width)) {
                            col.width = width;
                        }
                        return col;
                    })
                    .value();
                $dataRow = angular.element(dataRow);
                datas = _.chain($dataRow.children('td'))
                    .map(angular.element)
                    .map(function ($cell) {
                        return $cell.text();
                    })
                    .value();
                if (columns.length !== datas.length) {
                    throw new Error('解析出的表头列数与数据列数不同， 请检查传入的模板是否合法 :' + printTemplate);
                }
                for (i = 0, len = columns.length; i < len; i++) {
                    col = columns[i];
                    col.field = datas[i];
                }
                return columns
            };

            PrintSettingsProto._getOriginalPrintCols = function () {
                var me = this,
                    grid = me.grid,
                    originalPrintCols = grid.__originalPrintCols;
                if (_.isArray(originalPrintCols)) {
                    return originalPrintCols;
                } else {
                    var columns = grid.__originalCols || grid.columns;
                    return grid.__originalPrintCols = _.map(columns, function (column) {
                        return {
                            field: column.field,
                            text: column.text
                        };
                    });
                }
            };

            PrintSettingsProto.show = function () {
                var me = this;
                if (!(me.scope.$$phase || me.scope.$root.$$phase)) {
                    me.scope.$apply(function () {
                        var $element = me.element;
                        $element.show();
                        GillionMsg.mask($element);
                    });
                } else {
                    me.scope.$evalAsync(function () {
                        var $element = me.element;
                        $element.show();
                        GillionMsg.mask($element);
                    });
                }
            };

            PrintSettingsProto.hide = function () {
                this.element.hide();
                GillionMsg.unMask();
            };

            PrintSettingsProto.enter = function () {
                var me = this;
                newTmpl = me.parsePrintTmplCall.call(me, me.oldTemplate, me.selected);
                me.sededCallback(newTmpl);
                me.hide();
            };

            PrintSettingsProto._getSelected = function () {
                return this.selected;
            };

            PrintSettingsProto._getUnSelected = function () {
                var me = this,
                    selected = me.selected,
                    source = me.source;
                return _.without.apply(null, [source].concat(selected));
            };

            PrintSettingsProto._addSelection = function ($event, item, type) {
                var me = this,
                    selections = me.selections;
                if (!$event.ctrlKey) {
                    selections.L.length = 0;
                    selections.R.length = 0;
                }
                selections[type].push(item);
            };

            PrintSettingsProto._anySelection = function (item, type) {
                return _.contains(this.selections[type], item);
            };

            PrintSettingsProto._last = function () {
                var me = this,
                    selected = me.selected,
                    leftSelections = me.selections.L,
                    withoutSelection;
                withoutSelection = _.partial(_.without, selected).apply($window, leftSelections);
                me.selected = withoutSelection.concat(leftSelections);
            };

            PrintSettingsProto._down = function () {
                var me = this,
                    selected = me.selected,
                    leftSelections = me.selections.L,
                    i, item;
                for (i = (selected.length - 2); i >= 0; i--) {
                    item = selected[i];
                    if (_.contains(leftSelections, item)
                        && !_.contains(leftSelections, selected[i + 1])) {
                        selected.splice(i, 1);
                        selected.splice(i + 1, 0, item);
                    }
                }
            };
            PrintSettingsProto._up = function () {
                var me = this,
                    selected = me.selected,
                    leftSelections = me.selections.L,
                    i, len, item;
                for (i = 1, len = selected.length; i < len; i++) {
                    item = selected[i];
                    if (_.contains(leftSelections, item)
                        && !_.contains(leftSelections, selected[i - 1])) {
                        selected.splice(i, 1);
                        selected.splice(i - 1, 0, item);
                    }
                }
            };
            PrintSettingsProto._first = function () {
                var me = this,
                    selected = me.selected,
                    leftSelections = me.selections.L,
                    withoutSelection;
                withoutSelection = _.partial(_.without, selected).apply($window, leftSelections);
                me.selected = leftSelections.concat(withoutSelection);
            };

            PrintSettingsProto._deselect = function () {
                var me = this;
                me.selected = _.partial(_.without, me.selected).apply($window, me.selections.L);
                me.selections.L.length = 0;
            };

            PrintSettingsProto._deselectAll = function () {
                var me = this;
                me.selected.length = 0;
                me.selections.L.length = 0;
            };
            PrintSettingsProto._select = function () {
                var me = this;
                Arrays.pushAll(me.selections.R, me.selected);
                me.selections.R.length = 0;
            };
            PrintSettingsProto._selectAll = function () {
                var me = this;
                me.selected.length = 0;
                Arrays.pushAll(me.source, me.selected);
                me.selections.R.length = 0;
            };

            PrintSettingsProto.__buildPrintTmpl = function () {
             var me = this;
             return '$TEXT_S$LODOP.ADD_PRINT_TABLE("10mm","0mm",800,600,"tablehtml");$TEXT_E$ $TABLE_S$ <style type="text/css"> .print_table { border-collapse: collapse; border-spacing: 0; width: 277mm; } .print_table caption { font-size: 1.2em; padding: .5em 0; font-weight: bold; } .print_table thead td { font-size: 1em;} .print_table thead tr.sec td { padding-bottom: .8em; border-bottom: 1px solid #333; } .print_table td { padding: .5em; font-size: 12px; } .print_table tbody td { border: 1px solid #333; }      </style> <table class="print_table" cellpadding="0" cellspacing="0"><tbody><tr><td colspan="15" align="center" style="height: 35px;padding:0;border:0px solid #333;font-weight:bold;font-size:24px;"><span >CompanyName-提配清单</span></td></tr><tr><td style="border:0px;" colspan="15" align="right"><span>NO:pudplanNo</span></td></tr><tr><td colspan="2" style="border-right-color:#ffffff">车牌号:vehicleNo</td><td colspan="2" style="border-right-color:#ffffff">司机:driverName</td><td colspan="3" style="border-right-color:#ffffff">电话:driverTel</td><td colspan="8">车费:carfeeTotal</td></tr>$TABLE_T1S$<tr><td colspan="15">送货明细</td></tr>'
                 + headerTmpl(me) +
                 '$TABLE_T1E$$TABLE1_S$'+
                 dataRowTmpl(me) +
                 '$TABLE1_E$$TABLE_H1S$<tr><td colspan="12" style="border-right-color:#ffffff">合计:共 PickupCount1 票 PickupQuantity1 件</td><td colspan="3">收款合计:goodsArriveTotal</td></tr>$TABLE_H1E$';
             };

            return {
                restrict: 'E',
                template: '<div class="table-settings" id="pintSettings" style="display: none;">\n    <div class="title"><span class="fl">自定义列表项</span>\n        <button class="fr" ng-click="pintSettings.hide()"><i class="iconfont2 icon-guanbi2"></i></button>\n    </div>\n    <div class="body">\n        <div class="box fl">\n            <h3> 已显示列\n                <button ng-click="pintSettings._first()" type="button" class="fr">\n                    <i class="iconfont2 icon-dingbu"></i>\n                </button>\n                <button ng-click="pintSettings._up()" type="button" class="fr mt-2">\n                    <i class="iconfont2 icon-shangyi"></i>\n                </button>\n                <button ng-click="pintSettings._down()" type="button" class="fr mt-2">\n                    <i class="iconfont2 icon-xiayi"></i>\n                </button>\n                <button ng-click="pintSettings._last()" type="button" class="fr">\n                    <i class="iconfont2 icon-dibu"></i>\n                </button>\n            </h3>\n            <ul>\n                <li tabindex="{{$index}}"\n                    ng-class="{\'act\': pintSettings._anySelection(item, \'L\')}"\n                    ng-repeat="item in pintSettings._getSelected() track by $id(item)"\n                    ng-click="pintSettings._addSelection($event, item, \'L\')">\n                    <div class="tableSet-colName">{{item.text}}</div>\n                    <div class="tableSet-colWidth" ng-class="{\'show\': item.width}">\n                        <input type="number" ng-model="item.width"/>\n                        <span>px</span>\n                        <div class="tableSet-border"></div>\n                        <p>——</p>\n                    </div>\n                </li>\n            </ul>\n        </div>\n        <div class="center fl">\n            <a ng-click="pintSettings._deselect()" href="javascript:;"><i class="iconfont2 icon-xiangyou"></i></a> \n            <a ng-click="pintSettings._deselectAll()" href="javascript:;"><i class="iconfont2 icon-xiangyou2"></i></a> \n            <a ng-click="pintSettings._select()" href="javascript:;"><i class="iconfont2 icon-xiangzuo"></i></a> \n            <a ng-click="pintSettings._selectAll()" href="javascript:;"><i class="iconfont2 icon-xiangzuo2"></i></a>\n        </div>\n        <div class="box fr"><h3> 未显示列 </h3>\n            <ul>\n                <li ng-class="{\'act\': pintSettings._anySelection(item, \'R\')}"\n                    ng-repeat="item in pintSettings._getUnSelected() track by $id(item)"\n                    ng-click="pintSettings._addSelection($event, item, \'R\')">\n                    <div class="tableSet-colName">{{item.text}}</div>\n                    <div class="show tableSet-colWidth">\n                        <input type="text" disabled="" ng-value="item.width">\n                        <span>px</span>\n                        <div class="tableSet-border"></div>\n                    </div>\n                </li>\n            </ul>\n        </div>\n    </div>\n    <div class="footer">\n        <a href="javascript:;" class="btn btn-normal" ng-click="pintSettings.enter()">确认</a> \n        <a href="javascript:;" class="btn btn-stroke" ng-click="pintSettings.hide()">取消</a>\n    </div>\n</div> ',
                replace: true,
                scope: true,
                link: function (scope, element) {
                    scope.pintSettings = new PrintSettings(scope, element);
                }
            };
        })
        .factory('PrintSettings', function ($rootScope, $compile) {
            var mainWindow = getMainWindow(window),
                $dom, $mainBody;
            if (!mainWindow.__printSettings) {
                $dom = angular.element('<g-print-settings></g-print-settings>');
                $compile($dom)($rootScope);
                $mainBody = angular.element(mainWindow.document.body);
                $mainBody.append($dom);
                $mainBody.append('<style></style>');
                mainWindow.__printSettings = $dom.scope().pintSettings
            }
            return mainWindow.__printSettings;
        });
});