/**
 * Created by huangzx on 2015/6/16.
 */
define('framework/tab/GillionTabModule', [
    'angular',
    'css!../../../css/tab.css'
], function(angular) {
    //滚动条按钮显示判断
    function setScrollers($panel) {
        var headerWidth = $panel.find('.tab-header').width(),
            navWidth = 0;
        $panel.find('.tab-nav').children().each(function(){
            navWidth += $(this).outerWidth(true);
        });

        if (navWidth > headerWidth){
            $panel.find('.tab-nav-left').show();
            $panel.find('.tab-nav-right').show();
        } else {
            $panel.find('.tab-nav-left').hide();
            $panel.find('.tab-nav-right').hide();
        }
    }

    /**
     * 根据选择器获得对应封装对象TabPanel
     * @param selector {String} 已配置过g-tab-panel的id选择器
     * @constructor
     */
    function TabPanel(selector) {
        this.$panel = angular.isObject(selector)?selector:angular.element(selector);          //面板节点
        this.$navs = this.$panel.find('.tab-nav');      //标签导航父节点
        this.$tabs = this.$panel.find('.tab-body');     //标签内容父节点
        this.navs = {};                                     //缓存标签导航，以title为key
        this.tabs = {};                                     //缓存标签内容，以title为key
        var navs = this.$navs.children(),
            tabs = this.$tabs.children(),
            key = null;
        for (var i=0; i<navs.length; i++) {
            key = navs[i].find('.tab-nav-title');
            this.navs[key] = navs[i];
            this.tabs[key] = tabs[i];
        }
    }

    /**
     * 添加一个新标签页
     * @param options 配置项
     *  [options.title] {String} 标题
     *  [options.content] {String}  内容，支持html标签片段
     *  [options.closable] {boolean}  是否可关闭
     *  [options.onclose] {Function}  关闭时候回调方法
     */
    TabPanel.prototype.addTab = function(options) {
        options = options || {};
        var self = this,
            $panel = this.$panel,
            $navs = self.$navs,
            $tabs = self.$tabs,
            navs = self.navs,
            tabs = self.tabs,
            title = options.title,
            content = options.content || '',
            closable = options.closable,
            onclose = options.onclose,
            nav,                         //标签头
            tab = options.tab;        //标签体

        //title唯一，已存在则不添加
        if (navs[title] && tabs[title]) {
            tab && tab.remove();
            return;
        }

        //初始化标签头
        nav = angular.element('<li><span class="tab-nav-title">' + title + '</span></li>');
        closable && nav.append('&nbsp;<span class="close">×</span>') &&
        nav.on('click', '.close', function() {
            if (angular.isFunction(onclose)) {
                onclose() && self.closeTab(title);
            } else {
                self.closeTab(title);
            }
        });
        nav.on('click', '.tab-nav-title', function(){
            self.selectTab(title);
        });
        $navs.append(nav);
        navs[title] = nav;

        //初始化标签内容
        if (!tab) {
            tab = angular.element('<div class="tab-content"></div>');
            tab.html(content);
            $tabs.append(tab);
        }
        tabs[title] = tab;

        //添加完并选中该页签
        self.selectTab(title);
        setScrollers($panel);
    };

    /**
     * 关闭页签
     * @param title {String} 页签头
     */
    TabPanel.prototype.closeTab = function(title) {
        var nav = this.navs[title],
            tab = this.tabs[title],
            selectNav = nav && nav.next();  //关闭后新的标签导航
        if (!selectNav.html()) selectNav = nav.prev();
        if (nav.hasClass('select-nav') && selectNav.html()) {
            selectNav.find('.tab-nav-title').trigger('click');
        }
        nav.remove();
        tab.remove();
        delete this.navs[title];
        delete this.tabs[title];
        setScrollers(this.$panel);
    };

    /**
     * 刷新页签
     * @param title {String} 页签头
     */
    TabPanel.prototype.refreshTab = function(title) {
    };

    /**
     * 选中标签
     * @param title {String} 页签头
     */
    TabPanel.prototype.selectTab = function(title) {
        var $panel = this.$panel,
            $tabHeader = $panel.find('.tab-header'),
            nav = this.navs[title],
            tab = this.tabs[title],
            headerWidth = $tabHeader.width(),
            navWidth = 0;
        //显示选中标签同时隐藏其他标签页面
        nav && nav.addClass('select-nav').siblings().removeClass('select-nav');
        tab && tab.show().siblings().hide();

        nav.prevAll().add(nav).each(function() {
            navWidth += $(this).outerWidth(true);
        });
        if (navWidth > headerWidth) { //判断是否需要滚动
            $tabHeader.scrollLeft($tabHeader.scrollLeft() + navWidth - headerWidth);
        }
    };

    return angular.module('GillionTabModule', []).
        factory('GillionPanelService', function() {
            var tabPanelCache = {};     //缓存获取过的标签页面板
            return {
                /**
                 * @param selector 已配置过g-tab-panel的id选择器
                 * @returns {TabPanel}
                 */
                getTabPanel: function(selector){
                    if (!/^#[\w-]+$/.test(selector)) return null;                      //不匹配id选择器，返回null对象
                    if (tabPanelCache[selector]) return tabPanelCache[selector];        //缓存中获取
                    var ele = angular.element(selector);
                    if (!ele.hasClass('tab-panel')) return null;
                    tabPanelCache[selector] = new TabPanel(selector);
                    return tabPanelCache[selector];
                }
            };
        }).
        directive('gTabPanel', function() {
            return {
                restrict: 'E',
                scope:{
                    defaultTitle: '@'
                },
                transclude: true,
                replace:true,
                template: '<div class="tab-panel"><button class="tab-nav-left">←</button><button class="tab-nav-right">→</button><div class="tab-header"><ul class="tab-nav"></ul></div><div class="tab-body" ng-transclude></div></div>',
                controller:function($scope, $element) {
                    $scope.tabPanel = this.tabPanel = new TabPanel($element);
                    $(window).resize(function(){setScrollers($element);});  //窗体改变
                },
                link: function(scope, iEle, iAttrs) {
                    iEle.find('.tab-nav-left').click(function(){
                        iEle.find('.tab-header').scrollLeft(
                            iEle.find('.tab-header').scrollLeft()-50)
                    });
                    iEle.find('.tab-nav-right').click(function(){
                        iEle.find('.tab-header').scrollLeft(
                            iEle.find('.tab-header').scrollLeft()+50)
                    });

                    if (scope.defaultTitle) {   //选中标签
                        scope.tabPanel.selectTab(scope.defaultTitle);
                    } else {  //无配置则显示第一个标签
                        iEle.find('.tab-nav').find(':first-child > .tab-nav-title').trigger('click');
                    }
                }
            };
        }).
        directive('gTab', function(){
            return {
                restrict: 'E',
                scope:{
                    title:'@',
                    closable: '@',
                    onclose:'='
                },
                transclude: true,
                replace:true,
                require:'^gTabPanel',
                template: '<div class="tab-content" ng-transclude></div>',
                link: function(scope, iEle, iAttrs, gTabPanelController) {
                    var opt = {
                        title: scope.title,
                        closable: (scope.closable === 'true')?true:false,
                        onclose: scope.onclose || null,
                        tab: iEle
                    };
                    gTabPanelController.tabPanel.addTab(opt);
                }
            };
        });
});