define([
    'angular',
    '/static/config.properties.js'
], function (angular, config) {
    var uploaderConfig = config.controls.uploader,
        urls = uploaderConfig.urls,
        previewClassExtMap = uploaderConfig.previewClassExtMap,
        isIE = /msie/.test(navigator.userAgent.toLowerCase()),
        CONSTANTS = {
            IMG_EXTENSIONS: ['bmp', 'jpg', 'jpeg', 'png', 'tiff', 'gif', 'pcx', 'tga', 'exif', 'fpx', 'svg', 'psd', 'cdr', 'pcd', 'dxf', 'ufo', 'eps', 'ai', 'raw']
        };
    return function (Arrays) {
        return {
            /**
             * 判断文件是否是图片
             * @param fileName {String} 文件名
             */
            isImg: function (fileName) {
                var extension = this.getExtension(fileName);
                return Arrays.exists(CONSTANTS.IMG_EXTENSIONS, extension);
            },

            /**
             * 获取文件扩展名
             * @param fileName {String} 文件名
             */
            getExtension: function (fileName) {
                var index;
                if (angular.isString(fileName)) {
                    index = fileName.lastIndexOf('.');
                    if (index != -1) {
                        return fileName.substr(index + 1);
                    }
                }
                return '';
            },

            /**
             * 根据文件路径获取文件名
             * @param path {String} 文件路径
             */
            getFilename: function (path) {
                if (angular.isString(path)) {
                    var tokens = path.split(/\/|\\/);
                    if (tokens.length > 0) {
                        return tokens[tokens.length - 1];
                    }
                }
                return '';
            }
        };
    };
});