define('framework/uploader/UploaderModule', [
    'angular',
    'jquery',
    '/static/config.properties.js',
    'framework/msg/GillionMsgModule',
    'framework/imageViewer2/GillionImageViewerModule'
], function (angular, $, config) {
    var uploaderConfig = config.controls.uploader,
        ctx = config.$paths.$current.ctx,
        urls = uploaderConfig.urls,
        previewClassExtMap = uploaderConfig.previewClassExtMap,
        defaultShowDeleter = uploaderConfig.showDeleter,
        navAgent = navigator.userAgent.toLowerCase(),
        isIE = /msie/.test(navAgent),
        isWebkit = /webkit/.test(navAgent),
        isFirefox = /firefox/.test(navAgent),
        CONSTANTS = {
            IMG_EXTENSIONS: ['bmp', 'jpg', 'jpeg', 'png', 'tiff', 'gif', 'pcx', 'tga', 'exif', 'fpx', 'svg', 'psd', 'cdr', 'pcd', 'dxf', 'ufo', 'eps', 'ai', 'raw'],
            UPLOADER_STATE: {
                BLANK: 'BLANK',
                UPLOADING: 'UPLOADING',
                UPLOADED: 'UPLOADED'
            },
            UPLOAD_FILE_EXTRA_PREFIX: '$upload_file_extra_',
            DELETE_FILE_EXTRA_PREFIX: '$delete_file_extra_',
            SHOW_DELETER: {
                ALWAYS: 'always',
                NEVER: 'never',
                HOVER: 'hover'
            }
        };

    function makeEmpty(object) {
        angular.forEach(object, function (v, k) {
            delete object[k];
        });
    }

    function isEmptyFileInfo(fileInfo) {
        return angular.isObject(fileInfo) && !fileInfo.fileKey;
    }

    return angular.module('UploaderModule', ['GillionImageViewerModule'])
        .directive('gUploadGroup', function ($http, Arrays, Functions, Predicates, GillionMsg, ImageViewer) {

            var UploaderGroupProto = UploaderGroup.prototype,
                UploaderStrategyProto = UploaderStrategy.prototype,
                newEqFileInfoPredicate = function (fileInfo) {
                    if (angular.isObject(fileInfo)) {
                        return Predicates.or(Predicates.newEqPredicate(fileInfo), Predicates.newPropValEqPredicate('fileKey', fileInfo.fileKey));
                    }
                    return false;
                };

            /**
             *
             * @param config
             * @param [config.ngModel] {ngModelController} 上传控件的绑定模型
             * @param config.scope
             * @param config.$element dom
             * @param config.$attrs dom属性对象
             * @constructor
             */
            function UploaderGroup(config) {
                var me = this,
                    scope = me.scope = config.scope,
                    strategyId = scope.key;

                me.ngModel = config.ngModel;
                me.$attrs = config.$attrs;
                me.free = true; // 非空闲状态不可上传
                me.changeByInner = false;
                me.uploaderStrategy = new UploaderStrategy(strategyId, me);
                me.showDeleter = scope.$showDeleter || defaultShowDeleter || CONSTANTS.SHOW_DELETER.ALWAYS;
                if (scope.autoLoadByKey === 'true') {
                    $http.get(ctx + urls.getFileInfosByStrategyId + strategyId)
                        .success(function (fileInfos) {
                            me.syncFromOuter(fileInfos);
                        });
                }
            }

            UploaderGroupProto.getInitialFileInfos = function () {
                var me = this,
                    scope = me.scope,
                    $source = scope.source,
                    fileInfos = [],
                    initialShowCount = me.getInitialShowCount(),
                    i;
                angular.copy($source, fileInfos);
                if (fileInfos.length < initialShowCount) {
                    for (i = fileInfos.length; i < initialShowCount; i++) {
                        fileInfos.push({});
                    }
                }
                return fileInfos;
            };

            UploaderGroupProto.getInitialShowCount = function () {
                var initialShowCount = Number(this.scope.initialShowCount);
                if (angular.isNumber(initialShowCount) && initialShowCount > 1) {
                    return initialShowCount;
                }
                return 1;
            };

            UploaderGroupProto.showImages = function (fileInfo) {
                var me = this,
                    disablePreview = me.scope.disablePreview,
                    fileKeys;
                if (disablePreview !== 'true') {
                    fileKeys = Arrays.transform(me.scope.fileInfos, Functions.newPropTransformer('fileKey'));
                    fileKeys = Arrays.filter(fileKeys, angular.isDefined);
                    ImageViewer.show(fileInfo.fileKey, fileKeys);
                }
            };

            UploaderGroupProto.syncFromOuter = function (outer) {
                var me = this,
                    fileInfos = me.scope.fileInfos;
                if (angular.isArray(outer)) {
                    me.uploaderStrategy.then(function (uploaderStrategy) {
                        var different = Arrays.subtract(outer, fileInfos, newEqFileInfoPredicate),
                            countLimit = uploaderStrategy.countLimit,
                            len = fileInfos.length,
                            i = 0,
                            j;
                        for (j = 0; j < different.length; i++) {
                            if (i < len && isEmptyFileInfo(fileInfos[i])) {
                                fileInfos.splice(i, 1, different[j]);
                            }else {
                                fileInfos.push(different[j]);
                            }
                            j++;
                        }
                        if (fileInfos.length > countLimit && me.lastFileInfoIsEmpty()) {
                            fileInfos.splice(fileInfos.length - 1, 1);
                        } else {
                            me.addEmptyUploaderIfPassable();
                        }
                    });
                }
            };

            /**
             * 如果最后一个 `uploader` 不为空， 并且没达到上传数上限， 添加一个空的上传控件
             * @protected
             */
            UploaderGroupProto.addEmptyUploaderIfPassable = function () {
                var me = this;
                me.uploaderStrategy.then(function (uploaderStrategy) {
                    if (!(me.lastFileInfoIsEmpty() || uploaderStrategy.isHitCap())) {
                        me.changeByInner = true;
                        me.scope.fileInfos.push({});
                    }
                });
            };

            UploaderGroupProto.lastFileInfoIsEmpty = function () {
                var me = this,
                    fileInfos = me.scope.fileInfos,
                    last = fileInfos[fileInfos.length - 1];
                return isEmptyFileInfo(last);
            };

            function UploaderStrategy(uploaderStrategyId, uploaderGroup) {
                var me = this;
                me.uploaderGroup = uploaderGroup;
                me.thenCallbacks = [];
                me.inited = false;
                $http.get(ctx + urls.getStrategyPrefix + uploaderStrategyId).success(function (strategyConfig) {
                    me.init(strategyConfig);
                    angular.forEach(me.thenCallbacks, function (callback) {
                        callback(me);
                    });
                    delete me.thenCallbacks;
                });
            }

            UploaderStrategyProto.isHitCap = function () {
                var me = this,
                    scope = me.uploaderGroup.scope,
                    uploadedCount = Arrays.count(scope.fileInfos, Predicates.newPropPredicate('fileKey', angular.isString)),
                    countLimit = me.countLimit;
                return !(countLimit && countLimit > uploadedCount);
            };

            /**
             *
             * @param config
             * @param config.allowExtensions 允许的后缀名
             * @param config.countLimit 允许上传的最大数量
             * @param config.sizeLimit 允许上传的文件最大大小 单位：(MB)
             * @constructor
             */
            UploaderStrategyProto.init = function (config) {
                var me = this;
                me.allowExtensions = Arrays.transform(config.allowExtensions, Functions.funcRef(String.prototype.toLowerCase));
                me.countLimit = config.countLimit;
                me.kbSizeLimit = config.sizeLimit;
                me.byteSizeLimit = config.sizeLimit * 1024;
                me.inited = true;
            };

            /**
             * @async
             */
            UploaderStrategyProto.then = function (callback) {
                var me = this;
                if (me.inited === true) {
                    callback(me);
                } else {
                    me.thenCallbacks.push(callback);
                }
            };

            /**
             * @async
             */
            UploaderStrategyProto.checkFileType = function (extension) {
                var me = this;
                me.then(function () {
                    if (!me.isSupportedFileType(extension)) {
                        GillionMsg.alert("提示信息", "上传失败，上传的文件类型非法！");
                    }
                });
            };

            /**
             * @async
             */
            UploaderStrategyProto.checkFileSize = function (byteSize) {
                var me = this;
                me.then(function () {
                    var kbSizeLimit = me.kbSizeLimit;
                    if (!me.isSupportedFileSize(byteSize)) {
                        GillionMsg.alert("提示信息", "上传失败，上传的文件超过最大大小：" + kbSizeLimit + "KB！");
                    }
                })
            };

            UploaderStrategyProto.isSupportedFileType = function (extension) {
                return Arrays.exists(this.allowExtensions, extension.toLowerCase());
            };

            UploaderStrategyProto.isSupportedFileSize = function (byteSize) {
                if (!isIE) {
                    return byteSize <= this.byteSizeLimit;
                }
                return true;
            };

            return {
                template: '<div class="form-upload">\n    <g-uploader ng-repeat="fileInfo in fileInfos"></g-uploader>\n    <div ng-transclude></div>\n</div>',
                replace: true,
                restrict: 'E',
                transclude: true,
                scope: {
                    key: '@',
                    autoLoadByKey: '@',
                    source: '=ngModel',
                    onBeforeUpload: '&',
                    onUpload: '&',
                    onBeforeDelete: '&',
                    onDelete: '&',
                    uploadParams: '=',
                    deleteParams: '=',
                    initialShowCount: '@',
                    showFilename: '@',
                    $showDeleter: '@showDeleter',
                    keepUploaderOnRemove: '@',
                    disablePreview: '@'
                },
                require: '?ngModel',
                link: function (scope, element, attrs, ngModel) {
                    var uploaderGroup = scope.uploaderGroup = new UploaderGroup({
                        ngModel: ngModel,
                        scope: scope,
                        $element: element,
                        $attrs: attrs
                    });
                    scope.fileInfos = uploaderGroup.getInitialFileInfos();

                    scope.$watchCollection('source', function (newSource) {
                        if (uploaderGroup.changeByInner === false && angular.isArray(newSource) && newSource.length > 0) {
                            uploaderGroup.syncFromOuter(scope.fileInfos, newSource);
                        }
                        uploaderGroup.changeByInner = false;
                    });

                    scope.$on('uploader.complete', function ($event, fileInfo) {
                        if (!angular.isArray(scope.source)) {
                            scope.source = [];
                        }
                        uploaderGroup.changeByInner = true;
                        scope.source.push(fileInfo);
                        uploaderGroup.addEmptyUploaderIfPassable();
                        if (!scope.$$phase) {
                            scope.$digest();
                        }
                        //noinspection JSUnresolvedFunction
                        $event.stopPropagation();
                    });
                }
            }
        })
        .directive('gUploader', function ($http, Arrays, Files, GillionMsg) {
            var UploaderProto = Uploader.prototype;

            /**
             *
             * @param config
             * @param [config.ngModel] {ngModelController} 上传控件的绑定模型
             * @param config.scope
             * @param config.$element dom
             * @param config.$attrs dom属性对象
             * @constructor
             */
            function Uploader(config) {
                var me = this,
                    scope = me.scope = config.scope,
                    UPLOADER_STATE = CONSTANTS.UPLOADER_STATE;

                me.$element = config.$element;
                me.$attrs = config.$attrs;
                me.$previewImg = me.$element.find('.form-upload-btn-img img');
                me.fileInfo = scope.fileInfo;
                me.state = me.fileInfo.fileKey ? UPLOADER_STATE.UPLOADED : UPLOADER_STATE.BLANK;
                me.ifFreedBack();
                me.initUploaderDom();
                me.$file.bind('change', function () {
                    me.beginUpload();
                });
                switch (scope.uploaderGroup.showDeleter) {
                    case CONSTANTS.SHOW_DELETER.ALWAYS:
                        scope.showDeleter = true;
                        break;
                    case CONSTANTS.SHOW_DELETER.NEVER:
                        scope.showDeleter = false;
                        break;
                    case CONSTANTS.SHOW_DELETER.HOVER:
                        var deleter = me.$element.find('button.form-upload-btn-del');
                        me.$element.hover(
                            function () {
                                scope.showDeleter = true;
                                if (!scope.$$phase) {
                                    scope.$digest();
                                }
                            },
                            function () {
                                scope.showDeleter = false;
                                if (!scope.$$phase) {
                                    scope.$digest();
                                }
                            }
                        );
                        break;
                }
            }

            UploaderProto.ifFreedBack = function () {
                var me = this,
                    scope = me.scope,
                    fileInfo = me.fileInfo,
                    fileKey = fileInfo.fileKey,
                    UPLOADER_STATE = CONSTANTS.UPLOADER_STATE;
                if (fileKey) {
                    fileInfo.extension = Files.getExtension(fileInfo.name);
                    fileInfo.isImg = Files.isImg(fileInfo.extension);
                    if (!fileInfo.isImg) {
                        fileInfo.iconClass = getIconClass(fileInfo.extension);
                    } else {
                        me.$previewImg.attr('src', ctx + urls.downloadPrefix + fileKey);
                    }
                    me.state = UPLOADER_STATE.UPLOADED;
                } else {
                    me.state = UPLOADER_STATE.BLANK;
                }
            };

            UploaderProto.initUploaderDom = function setSubmitTargetName() {
                var me = this,
                    scope = me.scope,
                    $element = me.$element,
                    uploadUrl = ctx + urls.uploadPrefix + scope.key,
                    frameName = '$uploaderFrame' + scope.$id,
                    $form;
                me.$frame = angular.element('<iframe>', {
                    name: frameName,
                    style: 'display:none;'
                }).appendTo($element);

                $form = me.$form = angular.element('<form>', {
                    target: frameName,
                    action: uploadUrl,
                    method: 'post',
                    enctype: 'multipart/form-data',
                    style: 'display:none;',
                    html: '<input name="file" type="file"/>'
                }).appendTo($element);

                me.$file = $form.children('input:file');

                me.$frame[0].onload = function () {
                    me.completeUpload();
                };
            };

            UploaderProto.selectFile = function () {
                var me = this;
                me.$file.click();
            };

            UploaderProto.beginUpload = function () {
                var me = this,
                    scope = me.scope,
                    uploaderGroup = scope.uploaderGroup,
                    uploaderStrategy = uploaderGroup.uploaderStrategy,
                    fileInfo, uploadEventParams, onBeforeUpload;

                onBeforeUpload = scope.onBeforeUpload;
                fileInfo = getFileInfo(me.$file);
                uploaderStrategy.checkFileType(fileInfo.extension);
                uploaderStrategy.checkFileSize(fileInfo.byteSize);
                uploadEventParams = me.uploadEventParams = {
                    fileInfo: fileInfo,
                    uploaderStrategy: uploaderStrategy
                };
                if (onBeforeUpload(uploadEventParams) !== false) {
                    angular.extend(scope.fileInfo, fileInfo);
                    renderParamsToForm(scope.uploadParams, me.$form);
                    me.$attrs.$set('state', 'uploading');
                    me.renderPreview();
                    me.state = CONSTANTS.UPLOADER_STATE.UPLOADING;
                    me.$form.submit();
                    if (!scope.$$phase) {
                        try {
                            scope.$digest();
                        } catch (e) {
                        }
                    }
                }
            };

            UploaderProto.completeUpload = function () {
                var me = this,
                    scope = me.scope,
                    fileInfo = scope.fileInfo,
                    result = extractResponse(me.$frame),
                    errorMessages;
                if (result && result.success === false) {
                    errorMessages = result['errorMessages'];
                    if (angular.isArray(errorMessages) && errorMessages.length > 0) {
                        GillionMsg.alert("上传失败", errorMessages.join('<br/>'));
                    } else {
                        GillionMsg.alert("上传失败", "上传失败，请重新上传！");
                    }
                    makeEmpty(fileInfo);
                    me.state = CONSTANTS.UPLOADER_STATE.BLANK;
                } else {
                    angular.extend(fileInfo, result);
                    me.state = CONSTANTS.UPLOADER_STATE.UPLOADED;
                    scope.$emit('uploader.complete', me.fileInfo);
                    me.$element.removeAttr('state');
                }
                resetForm(me.$form);
                if (!scope.$$phase) {
                    scope.$digest();
                }
            };

            UploaderProto.renderPreview = function () {
                var me = this,
                    scope = me.scope,
                    fileInfo = scope.fileInfo,
                    extension = fileInfo.extension,
                    imgDom, fileDom, fileInstance, imagePath;
                if (Files.isImg(extension)) {
                    fileDom = me.$file[0];
                    if (fileDom.files && (fileInstance = fileDom.files[0])) {
                        var reader = new FileReader();
                        reader.onload = function (evt) {
                            me.$previewImg.attr('src', evt.target.result);
                        };
                        reader.readAsDataURL(fileInstance);
                    } else {
                        me.$previewImg[0].select();
                        imagePath = document.selection.createRange().text;
                        me.$previewImg.css('filter', "progid:DXImageTransform.Microsoft.AlphaImageLoader(sizingMethod='scale',src='" + imagePath + "')")
                    }
                } else {
                    fileInfo.iconClass = getIconClass(extension);
                }
            };

            UploaderProto.deleteFile = function (fileInfo) {
                var me = this,
                    scope = me.scope,
                    uploaderGroup = scope.uploaderGroup,
                    fileInfos = scope.fileInfos,
                    params = {};
                angular.forEach(scope.deleteParams, function (value, key) {
                    params[CONSTANTS.DELETE_FILE_EXTRA_PREFIX + key] = value;
                });
                $http.delete(ctx + urls.deletePrefix + fileInfo.fileKey, {
                    params: params
                }).success(function () {
                    if (scope.keepUploaderOnRemove) {
                        makeEmpty(fileInfo);
                        me.state = CONSTANTS.UPLOADER_STATE.BLANK;
                    } else {
                        uploaderGroup.changeByInner = true;
                        Arrays.remove(fileInfos, fileInfo);
                        uploaderGroup.addEmptyUploaderIfPassable();
                    }
                }).error(function (data) {
                    if (angular.isObject(data) && data.message) {
                        GillionMsg.alert('删除文件失败', data.message);
                    }
                });
            };

            function extractResponse($frame) {
                var $body = angular.element($frame[0].contentDocument.body),
                    responseJson;
                if ($body.is(':has(pre)')) {
                    responseJson = $body.children('pre').html();
                } else {
                    responseJson = $body.html();
                }
                responseJson = $.trim(responseJson);
                return $.parseJSON(responseJson);
            }

            /**
             * @private
             */
            function pairToUploadParamField(key, value) {
                var fieldName = CONSTANTS.UPLOAD_FILE_EXTRA_PREFIX + key,
                    fieldType;
                if (angular.isString(value)) {
                    fieldType = 'text';
                } else if (angular.isDate(value)) {
                    fieldType = 'date';
                } else if (angular.isNumber(value)) {
                    fieldType = 'number';
                }
                return angular.element('<input>', {
                    name: fieldName,
                    type: fieldType,
                    value: value
                });
            }

            /**
             * @private
             */
            function uploadParamsToFields(uploadParams) {
                var fieldsContainer = angular.element('<div data-params-container></div>');
                angular.forEach(uploadParams, function (value, key) {
                    var $paramField = pairToUploadParamField(key, value);
                    fieldsContainer.append($paramField);
                });
                return fieldsContainer;
            }

            function renderParamsToForm(uploadParams, $form) {
                var $paramsContainer = uploadParamsToFields(uploadParams);
                $form.append($paramsContainer);
            }


            function getIconClass(extension) {
                var iconClass = '';
                angular.forEach(previewClassExtMap, function (mapExtensions, className) {
                    if (Arrays.exists(mapExtensions, extension)) {
                        iconClass = className;
                        //noinspection UnnecessaryReturnStatementJS
                        return;
                    }
                });
                return iconClass;
            }


            function getFileInfo($file) {
                var fileInfo = {};
                fileInfo.path = $file.val();
                fileInfo.name = Files.getFilename(fileInfo.path);
                fileInfo.extension = Files.getExtension(fileInfo.name);
                fileInfo.isImg = Files.isImg(fileInfo.name);
                if (!isIE) {
                    fileInfo.byteSize = $file[0].files[0].size;
                }
                return fileInfo;
            }

            function resetForm($form) {
                var formDom = $form[0];
                if (formDom) {
                    formDom.reset();
                    $form.children('div[data-params-container]').remove();
                }
            }

            return {
                template: '<div class="form-upload-item">\n    <button class="btn form-upload-btn-add" ng-show="uploader.state === \'BLANK\'" ng-click="uploader.selectFile()">\n        <i class="fi fi-add"></i>\n        <span>上传文件</span>\n    </button>\n    <button class="btn form-upload-btn-del" ng-show="showDeleter && uploader.state === \'UPLOADED\'" ng-click="uploader.deleteFile(fileInfo)">\n        <i class="fi fi-close-xs"></i>\n    </button>\n    <div class="form-upload-icon" ng-show="uploader.state !== \'BLANK\'">\n        <button class="btn form-upload-btn-icon" ng-show="fileInfo.isImg === false">\n            <i class="fi" ng-class="fileInfo.iconClass"></i>\n        </button>\n        <button class="btn form-upload-btn-img" ng-show="fileInfo.isImg === true" ng-click="uploaderGroup.showImages(fileInfo)">\n            <img width="78" height="78">\n        </button>\n        <div class="form-upload-item-mask" ng-show="uploader.state === \'UPLOADING\'"></div>\n        <div class="form-upload-item-uploading" ng-show="uploader.state === \'UPLOADING\'">上传中</div>\n    </div>\n    <div class="form-upload-item-text" ng-show="uploader.state === \'UPLOADED\'" ng-if="showFilename !== \'false\'">\n        {{fileInfo.name}}\n    </div>\n</div>',
                replace: true,
                restrict: 'E',
                scope: false,
                link: function (scope, element, attrs, uploaderGroup) {
                    scope.uploader = new Uploader({
                        scope: scope,
                        $element: element,
                        $attrs: attrs
                    });
                }
            };
        })
        .factory('Files', function (Arrays) {

            return {
                /**
                 * 判断文件是否是图片
                 * @param fileName {String} 文件名
                 */
                isImg: function (fileName) {
                    var extension = this.getExtension(fileName);
                    return Arrays.exists(CONSTANTS.IMG_EXTENSIONS, extension.toLowerCase());
                },

                /**
                 * 获取文件扩展名
                 * @param fileName {String} 文件名
                 */
                getExtension: function (fileName) {
                    var index;
                    if (angular.isString(fileName)) {
                        index = fileName.lastIndexOf('.');
                        if (index != -1) {
                            return fileName.substr(index + 1);
                        } else {
                            return fileName;
                        }
                    }
                    return '';
                },

                /**
                 * 根据文件路径获取文件名
                 * @param path {String} 文件路径
                 */
                getFilename: function (path) {
                    if (angular.isString(path)) {
                        var tokens = path.split(/\/|\\/);
                        if (tokens.length > 0) {
                            return tokens[tokens.length - 1];
                        } else {
                            return path;
                        }
                    }
                    return '';
                }
            };
        });
});