# 上传控件开发文档

## 后端服务

> **概述** ：
> 上传控件后端服务由两部分支持
> - **gillion-cloud-filesystem** 文件系统API，所有文件操作通过该组件访问。
> - **gillion-${implement}-filesystem** 文件系统实现， 目前实现有 **gillion-local-filesystem**(用于开发) 与 **gillion-oss-filesystem**(用于上传到阿里云)。
> - 通过 **上传控件策略(UploaderStrategy)** 记录管理， 建议一个控件策略对应独立的前端控件实例。 通过 **上传控件策略(UploaderStrategy)** 可限制上传文件类型、大小、数量、自动生成微缩图。 且相关限制前后端统一。
> - 提供 **上传前处理器(BeforeUploadHandler)** 可在上传前修改文件和文件信息。
> - 提供 **事件监听(FileSystemAction)** 可单独或全部监听上传、下载、删除操作。
> - 提供 **文件信息服务(FileInfoService)** 、 **上传控件策略服务(UploaderStrategyService)** 和 **文件系统管理服务(CloudFileSystemManager)** 可根据需求操作文件。 
> - 文件信息统一通过 **文件键(fileKey)** 管理， 上传后能取得， 删除、 下载源文件、微缩图， 通过一致的服务即可完成。
> - 下载支持 **oss eTag** 缓存机制

### 部署

1. 在程序中加入依赖

```xml
<dependency>
    <groupId>com.gillion</groupId>
    <artifactId>gillion-local-filesystem</artifactId>
    <version>1.0.0.0.RELEASE</version>
</dependency>
<dependency>
    <groupId>com.gillion</groupId>
    <artifactId>gillion-cloud-filesystem</artifactId>
    <version>1.0.0.2.RELEASE</version>
</dependency>
```

> **NOTE** : 上面的依赖是开发环境示例， 具体请根据用途加入最新版本的依赖。

2. 导入 **Spring 配置文件**

- `applicationContext.xml` :

```xml
<import resource="classpath*:spring-context.xml"/>

<util:properties id="cloudFileSystemProps" location="classpath:properties/cloud_filesystem.properties"/>
```

- `dispatcher-servlet.xml` :

```xml
<import resource="classpath*:spring-mvc.xml"/>
```

- `properties/cloud_filesystem.properties`

```properties
filesystem.jdbcTemplate.beanName=jdbcTemplate
filesystem.ossClient.endpoint=http://oss-cn-hangzhou.aliyuncs.com
filesystem.ossClient.accessKeyId=0S1m6dsfWIp7O1PY
filesystem.ossClient.secretAccessKey=mrs93dfsfgMRBC52znVbxznELHiioLY
```

> **NOTE** :
> * `spring-context.xml` 和 `spring-mvc.xml` 如果因为文件名冲突等原因不能直接引入时， 请将jar包内的文件 copy出来重命名后导入
> * `cloud_filesystem.properties` 根据具体实现而定， 例如 `gillion-local-filesystem` 实现只需要 `filesystem.jdbcTemplate.beanName` 配置项即可。 实现jar包中有该文件模板。

## 前端控件

> **概述**
> - 此次改善尽量在不更改原API的前提下， 优化实现、改进功能。
> - 通用配置在 `config.properties.js` 可以统一配置， 若干配置可在控件上重写。
> - ~~通过 `ngModel` 可双向绑定文件~~， **但建议通过上传控件策略自动读取**。
> - 图片可点击进入预览页面
> - 控件预定义了部分类型的文件图标样式，可在 `config.properties.js` 中新增全局的文件图标样式 

## 控件全局配置

```js
define({
    ...
    controls: {
        ...
        uploader: {
            showDeleter: 'always',  // 何时显示删除按钮， 枚举型 'always' 始终, 'never' 从不, 'hover' 鼠标悬浮在图片时。 该设置可在控件重写
            urls: {
                uploadPrefix: '/cloud/filesystem/uploadFile/',                  //上传服务地址
                downloadPrefix: '/cloud/filesystem/downloadFile/',              //下载服务地址
                deletePrefix: '/cloud/filesystem/deleteFile/',                  //删除服务地址
                getStrategyPrefix: '/cloud/filesystem/getStrategy/',            //获取控件策略服务地址
                getFileInfosByStrategyId: '/cloud/filesystem/getFileInfos/'     //获取文件信息
            },
            unknownClass: 'fi-file-rar',  //当未定义文件类型图标样式时， 显示该样式
            previewClassExtMap: {  // 文件类型图标映射
                'fi-file-word': ['docx', 'dotx', 'doc', 'dot', 'docm', 'xps', 'rtf', 'wtf', 'odt'],
                'fi-file-pdf': ['pdf'],
                'fi-file-excel': ['xlsx', 'xlsm', 'xlsb', 'xltx', 'xltm', 'xls', 'xlt', 'xls', 'xml', 'xlam', 'xla', 'xlw', 'csv'],
                'fi-file-ppt': ['pptx', 'pptm', 'ppt', 'xps', 'potx', 'potm', 'pot', 'thmx', 'pps', 'ppsm', 'ppam', 'ppa'],
                'fi-file-rar': ['rar', 'zip', 'zz', 'zix', 'zipx', 'z', 'yz', 'war', 'tgz', 'rpm', 'rz', 'jar', 'gzip']
            }
        }
    },
    $paths: {
        $current: { // 项目上下文根， 必要
            ctx: '/gillion-web',
            path: ''
        },
        ...
    }
    ...
});
```

### 控件属性

Attribute Name        | Type                                           | Value Demo                | Desc
--------------------- | ---------------------------------------------- | ------------------------- | ----------------------------------------------------
key                   | String                                         | 1                         | **上传控件策略(UploaderStrategy)** 键
autoLoadByKey         | Boolean                                        | true                      | 是否根据 **上传控件策略(UploaderStrategy)** 自动显示文件
ngModel               | Array<Object>                                  | []                        | 绑定文件信息
onBeforeUpload        | function(fileInfo, uploaderStrategy):Boolean   | ...                       | 上传前事件， 返回 `false` 时不上传
onUpload              | function(fileInfo, uploaderStrategy)           | ...                       | 上传事件
onBeforeDelete        | function(fileInfo, uploaderStrategy):Boolean   | ...                       | 删除前事件， 返回 `false` 时不上传
onDelete              | function(fileInfo, uploaderStrategy)           | ...                       | 删除事件
uploadParams          | Object                                         | {rename: 'someNewFile'}   | 上传时得扩展参数， 可在 `handler` 、 `listener` 中使用
deleteParams          | Object                                         | {logic: true}             | 上传时得扩展参数， 可在 `handler` 、 `listener` 中使用
initialShowCount      | Number                                         | 5                         | 初始显示几个上传框
showFilename          | Boolean                                        | false                     | 上传后是否显示文件名
showDeleter           | Boolean                                        | `hover`                   | 何时显示删除按钮， 可能为：枚举型 `always` 始终, `never` 从不, `hover` 鼠标悬浮在图片时
keepUploaderOnRemove  | Boolean                                        | true                      | 删除时是否删除上传款, 默认删除
disablePreview        | Boolean                                        | true                      | 是否禁用图片预览
disableUpload         | Boolean                                        | true                      | 是否禁用上传, **NOTE** : 只是不会生成空上传框

### 图片预览

> NOTE : 各项目组请在 `static/app/framework/imageViewer2/viewer.html` 中写入 `<script type="text/javascript"> window.ctx="${当前项目上下文根}"</script>` 并锁掉该文件避免升级时覆盖。
> 调用服务 `ImageViewer.show(activeFileKey:String, fileKeys:Array<String>)` 可预览图片

## 示例

### 上传时根据重写文件名
- html
```html
<g-upload-group key="1"
                initial-show-count="1"
                upload-params="uploadParams"
                ng-model="fileInfos"
                auto-load-by-key="true">
    </g-upload-group>

    <input type="hidden" name="rename" ng-model="uploadParams.rename" ng-value="营业执照">
```

- handler

```java
package com.gillion.sebusiness.demo.service.upload.impl;

import com.gillion.upload.entity.FileUploadRequest;
import com.gillion.upload.entity.UploaderStrategy;
import com.gillion.upload.handlers.BeforeUploadHandler;
import org.apache.commons.io.FilenameUtils;
import org.springframework.stereotype.Component;

@Component
public class RenameUploadHandler implements BeforeUploadHandler {
    @Override
    public void call(FileUploadRequest request) {
        if (request.getUploadExtraData().containsKey("rename")) {
            final String originalName = request.getFileName();
            final String rename = (String) request.getUploadExtraData().get("rename");
            request.setFileName(rename + '.' + FilenameUtils.getExtension(originalName));
        }
    }

    @Override
    public boolean handle(UploaderStrategy strategy) {
        return true;
    }
}

```

### 会员页头像

> **需求:**
> 不支持预览
> 不显示文件名
> 鼠标移到图片时才显示删除按钮

- html
```html
<g-upload-group key="1"
                initial-show-count="1"
                disable-preview="true"
                show-deleter="hover"
                ng-model="fileInfos"
                auto-load-by-key="true"
                show-filename="false"/>
</g-upload-group>
```

### 图片查看

> **需求:**
> 不支持预览
> 不生成空上传控件
> 不现实删除按钮


- html
```html
<g-upload-group key="1"
                disable-preview="true"
                disable-upload="true"
                show-deleter="never"
                ng-model="fileInfos"
                auto-load-by-key="true"
                show-filename="true"/>
</g-upload-group>
```
