define(["angular",
    "framework/utils/DBCUtils"
], function (angular, DBCUtils) {
    /*
     * 校验输入字符串是否符合要求
     * val： 输入数字
     * s1：整数位数
     * s2：小数位数
     */
    var checkVal = function (val, s1, s2) {
        var rs, pos, dig;
        rs = val.indexOf(".");
        if (rs > 0) {
            pos = val.substring(0, rs);
            dig = val.substring(rs + 1, val.length);
        } else {
            pos = val.substring(0, val.length);
            dig = "";
        }
        if (isNaN(pos)) return false;
        if (rs > 0 && isNaN(dig)) return false;
        if (pos.length <= s1 && pos >= 0) {
            if (dig == "" || dig.length < s2) {
                return true;
            } else {
                return false
            }
        } else {
            return false
        }
    };

    /*
     * 替换非数字字符
     * val： 输入数字
     * emptyLeftZero：0不能在第一位输入
     */
    var repalceAllChar = function (val, emptyLeftZero) {
        if (undefined == val || val == "") return "";
        var ar = val.split('');
        if (parseFloat(val) != val || val.indexOf(".") >= 0) {
            for (var i = 0; i < ar.length; i++) {
                var charCode = ar[i].charCodeAt(0);
                if (charCode > 57 || charCode < 47) {
                    val = val.replace(ar[i], '');
                }
            }
        } else {
            if (undefined != emptyLeftZero && emptyLeftZero) {
                for (var i = 0; i < ar.length; i++) {
                    if (ar[0].charCodeAt(0) == 48) {
                        val = val.replace(ar[0], '');
                    } else {
                        break;
                    }
                }
            }
        }
        return val;
    };
    return {
        /*
         * 数字输入控制,只能输入数字
         * $event: 输入事件
         * s1：整数位数
         * s2：小数位数
         * emptyLeftZero：0不能在第一位输入
         */
        numToFloat: function ($event, s1, s2, emptyLeftZero) {
            var element, val, keyCode;
            element = angular.element($event.srcElement || $event.target);
            val = DBCUtils.toCDB($.trim(element.val()));
            if (val != '') {
                // 如果输入都是数字，且符合要求直接返回
                if (checkVal(val, s1, s2)) {
                    return val;
                }
                var rs, pos, dig;
                rs = val.indexOf(".");
                if (rs > 0) {
                    pos = repalceAllChar(val.substring(0, rs), emptyLeftZero);
                    dig = repalceAllChar(val.substring(rs, val.length), emptyLeftZero);
                } else {
                    pos = repalceAllChar(val.substring(0, val.length), emptyLeftZero);
                    dig = "";
                }
                if (undefined != pos && pos.length > s1) {
                    pos = pos.substring(0, s1);
                }
                if (undefined != dig && dig.length > s2) {
                    dig = dig.substring(0, s2);
                }
                if (pos.length == 0 && dig.length == 0) {
                    val = "";
                } else if (pos.length == 0 && dig.length != 0) {
                    val = "0." + dig;
                } else if (pos.length != 0 && dig.length == 0 && rs > 0) {
                    val = pos + ".";
                } else if (pos.length != 0 && dig.length == 0) {
                    val = pos;
                } else if (pos == 0 && dig.length == parseFloat(s2) && parseFloat(dig) == 0) {
                    val = "";
                } else if (pos.length != 0 && dig.length != 0) {
                    val = pos + "." + dig;
                }
            }
            element.val(val);
            return val;
        },

        /*
         * 数字输入控制,只能输入数字,返回一个整数
         * $event: 输入事件
         * s1：整数位数
         * emptyLeftZero：0不能在第一位输入
         */
        numToInt: function ($event, s1, emptyLeftZero) {
            var element, val, keyCode;
            element = angular.element($event.srcElement || $event.target);
            val = DBCUtils.toCDB($.trim(element.val()));
            if (val != "") {
                var ar = val.split('');
                if (parseFloat(val) != val || val.indexOf(".") >= 0) {
                    for (var i = 0; i < ar.length; i++) {
                        var charCode = ar[i].charCodeAt(0);
                        if (charCode > 57 || charCode < 47) {
                            val = val.replace(ar[i], '');
                        }
                    }
                } else {
                    if (undefined == emptyLeftZero || emptyLeftZero) {
                        for (var i = 0; i < ar.length; i++) {
                            if (ar[0].charCodeAt(0) == 48) {
                                val = val.replace(ar[0], '');
                            } else {
                                break;
                            }
                        }
                    }
                }
            }
            if (val.length > s1) {
                val = val.substring(0, s1);
            }
            element.val(val);
            return val;
        },

        /*
         * 判断键盘输入的是否是方向键
         * $event：输入事件
         */
        isDirectionKey: function ($event) {
            var keyCode = $event.keyCode;
            if (keyCode >= 37 && keyCode <= 40 || keyCode == 8) {
                return;
            }
        }
    }
});