# L2.1.12.1.B & Q2.1.5.3.R

更新内容有：
------------------------------------------------------------------------------------------------------------------------
1. 校验支持默认错误消息
2. 校验消息支持参数代入
3. 多字段联合校验支持任其一值更改时触发
    * `@Compare` 监听 `property` 和 `compareTo` 字段
    * `@AllMatch` 监听 `fields` 配置的所有字段
    * `@EitherNotBlank` 监听 `property` 和 `otherProperty` 字段
4. 新增日期大小校验 `@DateMax` 和 `@DateMin`
    + 支持固定日期校验
    + 支持以当前日期偏移量校验
5. 新增日期跨度校验 `@DateSpan`
6. 支持默认分组(默认分组不支持对象组合内的校验)
7. 校验规则文件和校验消息国际化文件现在生成到工程的`webapp`目录下，`web`工程与前端资源部署到不同服务器时， 不需要拷贝上述文件至前端资源目录

详情：
------------------------------------------------------------------------------------------------------------------------
### 1. 校验支持默认错误消息
+ 校验默认错误消息的`Key`为： `规则名.message`。
+ 以`@Require`必填为例， 在`i18n_messages`表插入如下数据，则所有`@Require`校验在没有配置错误消息时显示 *该字段必填*:

|key               | locale_id   | message                |
|----------------- | ----------- | ---------------------- |
|Require.message   | 1           | 该字段必填               |
|Require.message   | 2           | This field is require  |

### 2. 校验消息支持参数代入
+ 参数代入支持字段名称代入(只代入一个字段) 和 校验注解属性代入
+ 以 `@Range` 校验为例， 实体中校验规则声明如下 :
```java
public class Employee {
    @Range(min= 18, max= 60, groups= Order.Save.class)
    private Integer age;
    //...
}
```
+ 在 `i18n_messages` 表插入如下数据时， 生成中文错误消息 : _年龄必须在18和60之间_:

|key              | locale_id   | message                    |
|---------------- | ----------- | -------------------------- |
|Range.message    | 1           | {1}必须在{3}和{2}之间        |
|age              | 1           | 年龄                        |


+ **解析**：
     1. 假如`Employee`是`Order`类的组合`creator`, 最高优先级错误消息`key`为`Range.order.creator.age`
     2. 当`Range.order.creator.age`在`i18n_messages`不存在时取`Range.message`
     3. `{1}` 下标为1的代入参数是字段名称， 以默认 `key` 去掉规则名称后按级递减并尝试取值, 下面是按优先级排序的取字段 `key`
           + `order.creator.age`
           + `creator.age`
           + `age`
     4. 其它代入值为校验 `Annotation` 除了 `groups`, `message`, `payload` 的属性， 按照属性名称字符串默认顺序排序


### 3. 多字段联合校验支持任其一值更改时触发

 在生成规则时，为规则 `Map` 添加 `listenProperties` 属性重写监听属性, 可监听多个字段，

 监听属性优先级为:
   + `listenProperties`
   + `property`
   + `properties`


 例如自定义校验 `RequireOnOtherNotEmpty` 的 `constraintProcessor`
 ```java
 @Component
 public class RequireOnOtherNotEmptyConstraintProcessor extends AbstractTypeConstraintProcessor<RequireOnOtherNotEmpty> {
   @Override
   public Map<String, Object> generateRule(RequireOnOtherNotEmpty annotation) {
       final HashMap<String, Object> rule = Maps.newHashMap();
       rule.put("property", annotation.property());
       rule.put("otherProperty", annotation.otherProperty());
       rule.put( "ruleName", "requireOnOtherNotEmpty");
       // 如果不加入 listenProperties 默认只监听 `property` 字段
       // 当 `otherProperty` 字段更改时， 检验结果不会更改， 不能实时显示校验结果。
       rule.put( "listenProperties", new String[]{annotation.property(), annotation.otherProperty()});
       return rule;
   }
 }
 ```


### 4. 新增日期大小校验 `@DateMax` 和 `@DateMin`
+ 这两个校验属性一致，`@DateMax` 和 `@DateMin` 均为字段级校验， 属性列表如下:

|Attribute Name | Type                         | Default Value | Description                                     |
|---------------|------------------------------|---------------|-------------------------------------------------|
|value          | String                       | Current Date  | 有该值时为固定日期校验， 偏移仍可定义但毫无意义          |
|parsePattern   | String                       | "yyyy-MM-dd"  | 固定日期字符串的解析表达式                           |
|shiftValues    | [@ShiftValue](#shiftValue)[] | null          | 以当前日期或固定的日期偏移， 作为实际要比较的值          |


 **固定日期大小校验** :
    ```java
    public class Employee {
        // 员工入职日期最小是公司开业日期 `2010-08-15`
        @DateMin("2010-08-15")
        private Date inDay; // 入职日期
    }
    ```

 **偏移日期大小校验** :

 + 偏移日期大小校验基准时间为当日日期即 '年-月-日'
 + 当我们希望以当前日期为基准， 偏移若干时间时使用此属性 属性值为注解 `@ShiftValue`， <span id="shiftValue">@ShiftValue</span>注解属性如下:

Attribute Name | Type          | Default Value | Description
---------------|---------------|---------------|-------------------------------------------------
calendarField  | CalendarField | null          | 历法字段 可能为：(`YEARS`/`MONTHS`/`WEEKS`/`DAYS`/`HOURS`/`MINUTES`/`SECONDS`/`MILLISECONDS`)
amount         | int           | 1             | 偏移量

 示例：
 ```java
 public class Employee {

     // 新增时出生日期必须是 18 ~ 65 年前
     @DateMax(shiftValues = @ShiftValue(calendarField = CalendarField.YEARS, amount = -18))
     @DateMin(shiftValues = @ShiftValue(calendarField = CalendarField.YEARS, amount = -65))
     private Date birthDay;        // 生日

     // 结工资时保存下个工资结账日在未来一个月内
     @DateMax(shiftValues = @ShiftValue(calendarField = CalendarField.MONTHS), groups = Employee.Payoff.class)
     @DateMin(groups = Employee.Payoff.class)
     private Date nextPayoffDate;  // 下次发工资日

     // 下次提交月报日期为一个月以内， 但不包含一月后的日期
     @DateMax(shiftValues = {
             @ShiftValue(calendarField = CalendarField.MONTHS),
             @ShiftValue(calendarField = CalendarField.SECONDS, amount = -1)
     }, groups = CommitReport.class)
     private Date nextCommitReportDate;
 }
 ```


### 5. 新增日期跨度校验 `@DateSpan`

`@DateSpan` 为类级校验， 用于校验两个日期之间的时间跨度， 属性列表如下

Attribute Name | Type                          | Default Value | Description
---------------|-------------------------------|---------------|-------------------------------------------------
minDateField   | String                        | null          | 比较小的日期的字段名称
maxDateField   | String                        | null          | 比较大的日期的字段名称
limit          | [@ShiftValue](#shiftValue)[]  | null          | 最大跨度值

**示例**

订单搜索时， 创建日期区间不能超过一个月， 否则数据量太大会导致数据库性能严重降低。

```java
@DateSpan(
    minDateField= "createDateStart",
    maxDateField= "createDateEnd",
    limit= @ShiftValue(calendarField= CalendarField.MONTHS))
public class Order {

    private Date createDateStart;
    private Date createDateEnd;
    //...
}
```

*****

# L2.1.9.0.B & Q2.1.4.0.RELEASE

### Compare效验添加支持 **大于等于(GREATER_EQ)** 和 **小于等于(LESS_EQ)**

E:

```java
@Compare(property = "checkInAge", compareTo = "age", than = Compare.Than.LESS_EQ, groups = {Employee.Save.class, Employee.Update.class})
public class Employee extends BaseModel implements Serializable, Cloneable {
//...
}
```

